#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
明日方舟立绘下载程序启动脚本
提供简单的命令行界面来运行下载程序
"""

import asyncio
import argparse
# 兼容相对/绝对导入，支持脚本独立运行
try:
    from .illustration_downloader_v2 import ArknightsIllustrationDownloaderV2  # type: ignore
except Exception:
    from illustration_downloader_v2 import ArknightsIllustrationDownloaderV2  # type: ignore


def create_basic_config():
    """创建基础配置"""
    return {
        'download': {
            'max_concurrent': 3,
            'timeout': 60,
        },
        'filter': {
            'min_rarity': 1,
            'max_rarity': 6,
        },
        'output': {
            'create_subdirectories': True,
            'subdirectory_structure': {
                'by_rarity': True,
                'by_career': False,
                'by_type': True
            }
        }
    }


def create_high_quality_config():
    """创建高质量配置（只下载高星干员）"""
    return {
        'download': {
            'max_concurrent': 5,
            'timeout': 60,
        },
        'filter': {
            'min_rarity': 4,  # 只下载4星以上干员
            'max_rarity': 6,
        },
        'output': {
            'create_subdirectories': True,
            'subdirectory_structure': {
                'by_rarity': True,
                'by_career': True,
                'by_type': True
            }
        }
    }


def create_specific_career_config(careers):
    """创建特定职业配置"""
    return {
        'download': {
            'max_concurrent': 3,
            'timeout': 60,
        },
        'filter': {
            'min_rarity': 1,
            'max_rarity': 6,
            'include_careers': careers
        },
        'output': {
            'create_subdirectories': True,
            'subdirectory_structure': {
                'by_rarity': True,
                'by_career': True,
                'by_type': True
            }
        }
    }


def create_test_config():
    """创建测试配置（只下载少量干员进行测试）"""
    return {
        'download': {
            'max_concurrent': 2,
            'timeout': 30,
        },
        'filter': {
            'min_rarity': 6,  # 只下载6星干员
            'max_rarity': 6,
            'include_careers': ['近卫']  # 只下载近卫职业
        },
        'output': {
            'create_subdirectories': True,
            'subdirectory_structure': {
                'by_rarity': True,
                'by_career': True,
                'by_type': True
            }
        }
    }


async def main():
    """主函数"""
    parser = argparse.ArgumentParser(
        description="明日方舟干员立绘自动下载程序",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
使用示例:
  python run_illustration_download.py                    # 使用默认配置
  python run_illustration_download.py --basic            # 基础配置
  python run_illustration_download.py --high-quality     # 高质量配置（4星以上）
  python run_illustration_download.py --careers 近卫 狙击  # 特定职业
  python run_illustration_download.py --test             # 测试配置
  python run_illustration_download.py --output my_illustrations  # 自定义输出目录
        """
    )
    
    # 配置选项
    parser.add_argument('--basic', action='store_true', 
                       help='使用基础配置（推荐新手）')
    parser.add_argument('--high-quality', action='store_true',
                       help='使用高质量配置（只下载4星以上干员）')
    parser.add_argument('--careers', nargs='+', metavar='CAREER',
                       help='指定要下载的职业（如：近卫 狙击 术师）')
    parser.add_argument('--test', action='store_true',
                       help='使用测试配置（只下载少量干员进行测试）')
    
    # 输出选项
    parser.add_argument('--output', '-o', metavar='DIR',
                       help='指定输出目录（默认：illustrations）')
    
    # 下载选项
    parser.add_argument('--concurrent', '-c', type=int, metavar='NUM',
                       help='设置并发下载数（默认：3）')
    parser.add_argument('--timeout', '-t', type=int, metavar='SECONDS',
                       help='设置超时时间（默认：60秒）')
    
    # 过滤选项
    parser.add_argument('--min-rarity', type=int, metavar='RARITY',
                       help='设置最小稀有度（1-6）')
    parser.add_argument('--max-rarity', type=int, metavar='RARITY',
                       help='设置最大稀有度（1-6）')
    
    args = parser.parse_args()
    
    # 选择配置
    if args.test:
        config = create_test_config()
        print("使用测试配置：只下载6星近卫干员")
    elif args.high_quality:
        config = create_high_quality_config()
        print("使用高质量配置：只下载4星以上干员")
    elif args.careers:
        config = create_specific_career_config(args.careers)
        print(f"使用特定职业配置：只下载 {', '.join(args.careers)} 职业干员")
    elif args.basic:
        config = create_basic_config()
        print("使用基础配置")
    else:
        config = create_basic_config()
        print("使用默认配置")
    
    # 应用命令行参数
    if args.concurrent:
        config['download']['max_concurrent'] = args.concurrent
        print(f"设置并发数：{args.concurrent}")
    
    if args.timeout:
        config['download']['timeout'] = args.timeout
        print(f"设置超时时间：{args.timeout}秒")
    
    if args.min_rarity:
        config['filter']['min_rarity'] = args.min_rarity
        print(f"设置最小稀有度：{args.min_rarity}")
    
    if args.max_rarity:
        config['filter']['max_rarity'] = args.max_rarity
        print(f"设置最大稀有度：{args.max_rarity}")
    
    # 显示配置信息
    print("\n当前配置:")
    print(f"  并发数: {config['download']['max_concurrent']}")
    print(f"  超时时间: {config['download']['timeout']}秒")
    print(f"  稀有度范围: {config['filter']['min_rarity']}-{config['filter']['max_rarity']}")
    if 'include_careers' in config['filter']:
        print(f"  职业限制: {', '.join(config['filter']['include_careers'])}")
    print(f"  输出目录: {args.output or 'illustrations'}")
    
    # 确认开始下载
    print("\n准备开始下载...")
    try:
        input("按回车键开始下载，或按 Ctrl+C 取消...")
    except KeyboardInterrupt:
        print("\n下载已取消")
        return
    
    # 创建下载器并运行
    try:
        downloader = ArknightsIllustrationDownloaderV2(
            output_dir=args.output,
            config=config
        )
        await downloader.run()
    except KeyboardInterrupt:
        print("\n下载被用户中断")
    except Exception as e:
        print(f"\n下载过程中出现错误: {e}")
        print("请检查日志文件获取详细信息")


if __name__ == "__main__":
    try:
        asyncio.run(main())
    except KeyboardInterrupt:
        print("\n程序被用户中断")
    except Exception as e:
        print(f"\n程序运行出错: {e}")
        sys.exit(1)



import base64
from pathlib import Path
from jinja2 import Environment, FileSystemLoader
import zipfile
from typing import Optional, Dict
from nonebot_plugin_htmlrender import html_to_pic

# 设置Jinja2环境
env = Environment(
    loader=FileSystemLoader(Path(__file__).parent.parent / "resources" / "templates"),
    autoescape=True,
    enable_async=True
)

# 获取可写数据目录（与更新/下载/游戏读取一致）
def _safe_get_data_dir() -> Path:
    try:
        from nonebot_plugin_localstore import get_plugin_data_dir as _get_dir  # type: ignore
        return _get_dir()
    except Exception:
        return Path.home() / ".local" / "share" / "nonebot2" / "nonebot_plugin_arkguesser"

DATA_DIR = _safe_get_data_dir()
ILLUSTRATIONS_DIR = DATA_DIR / "illustrations"
IMAGES_DIR = DATA_DIR / "images"

def get_local_image_path(filename: str) -> str:
    """
    获取本地图片的完整路径
    
    Args:
        filename: 本地图片的文件路径或原文件名
    
    Returns:
        本地图片的文件路径或原文件名
    """
    if not filename:
        return ""
    
    # 检查是否是ZIP文件中的图片
    if "/" in filename and not filename.startswith(('http://', 'https://')):
        # 这是ZIP文件中的路径，格式如 "6/阿米娅.webp"
        return filename
    
    # 如果是其他格式的文件名，直接返回
    return filename

def get_zip_image_content(filename: str) -> Optional[bytes]:
    """
    从ZIP文件中获取图片内容
    
    Args:
        filename: ZIP文件中的图片路径，格式如 "6/阿米娅.webp"
    
    Returns:
        图片的二进制内容，如果失败则返回None
    """
    if not filename or "/" not in filename:
        return None
    
    try:
        # 构建ZIP文件路径 - 使用数据目录
        zip_path = IMAGES_DIR / "illustrations.zip"
        if not zip_path.exists():
            print(f"ZIP文件不存在: {zip_path}")
            return None
        
        with zipfile.ZipFile(zip_path, 'r') as zip_file:
            if filename in zip_file.namelist():
                return zip_file.read(filename)
            else:
                print(f"ZIP文件中未找到图片: {filename}")
                return None
                
    except Exception as e:
        print(f"读取ZIP文件失败 {filename}: {e}")
        return None

# 删除了random导入，不再需要随机背景图功能

def image_to_data_uri(image_path: str) -> str:
    """
    将本地图片转换为 data URI 格式
    
    Args:
        image_path: 图片路径或ZIP文件中的路径
    
    Returns:
        data URI 字符串、原路径或特殊标识符
    """
    if not image_path or image_path.startswith(('http://', 'https://')):
        # 如果是URL，直接返回
        return image_path
    
    # 检查是否是新的目录结构路径
    if "/" in image_path and not image_path.startswith(('http://', 'https://')) and not Path(image_path).exists():
        # 这是新的目录结构路径，格式如 "稀有度6/近卫/半身像/银灰_半身像_精英2.png"
        result = _new_illustration_to_data_uri(image_path)
        if not result:
            return "MISSING_IMAGE"  # 返回特殊标识符
        return result
    
    # 检查是否是ZIP文件中的图片路径
    if "/" in image_path and not Path(image_path).exists():
        # 这是ZIP文件中的路径，格式如 "6/阿米娅.webp"
        result = _zip_image_to_data_uri(image_path)
        if not result:
            return "MISSING_IMAGE"  # 返回特殊标识符
        return result
    
    # 处理本地文件
    try:
        image_file = Path(image_path)
        if not image_file.exists():
            print(f"图片文件不存在: {image_path}")
            return "MISSING_IMAGE"  # 返回特殊标识符
        
        # 检查文件大小
        file_size = image_file.stat().st_size
        MAX_SIZE = 3 * 1024 * 1024  # 3MB限制
        
        if file_size > MAX_SIZE:
            print(f"图片文件太大: {image_path} ({file_size / 1024:.1f} KB)")
            return "MISSING_IMAGE"  # 返回特殊标识符
        
        # 读取图片并转换为 base64
        with open(image_file, 'rb') as f:
            image_data = f.read()
        
        return _convert_image_data_to_uri(image_data, image_file.suffix)
        
    except Exception as e:
        print(f"转换图片失败 {image_path}: {e}")
        return "MISSING_IMAGE"  # 返回特殊标识符

def _new_illustration_to_data_uri(image_path: str) -> str:
    """
    将新目录结构中的立绘转换为 data URI 格式
    
    Args:
        image_path: 新目录结构中的图片路径，格式如 "稀有度6/近卫/半身像/银灰_半身像_精英2.png"
    
    Returns:
        data URI 字符串或空字符串
    """
    try:
        # 构建完整的文件路径（数据目录）
        full_path = ILLUSTRATIONS_DIR / image_path
        
        if not full_path.exists():
            print(f"新目录结构中的图片不存在: {full_path}")
            return "MISSING_IMAGE"
        
        # 检查文件大小
        file_size = full_path.stat().st_size
        MAX_SIZE = 3 * 1024 * 1024  # 3MB限制
        
        if file_size > MAX_SIZE:
            print(f"新目录结构中的图片太大: {image_path} ({file_size / 1024:.1f} KB)")
            return "MISSING_IMAGE"
        
        # 读取图片并转换为 base64
        with open(full_path, 'rb') as f:
            image_data = f.read()
        
        return _convert_image_data_to_uri(image_data, full_path.suffix)
        
    except Exception as e:
        print(f"转换新目录结构图片失败 {image_path}: {e}")
        return ""

def _zip_image_to_data_uri(zip_path: str) -> str:
    """
    将ZIP文件中的图片转换为 data URI 格式
    
    Args:
        zip_path: ZIP文件中的图片路径，格式如 "6/阿米娅.webp"
    
    Returns:
        data URI 字符串或空字符串
    """
    try:
        # 从ZIP文件中获取图片内容
        image_data = get_zip_image_content(zip_path)
        if not image_data:
            return "MISSING_IMAGE"
        
        # 检查数据大小
        if len(image_data) > 3 * 1024 * 1024:  # 3MB限制
            print(f"ZIP中的图片太大: {zip_path} ({len(image_data) / 1024:.1f} KB)")
            return "MISSING_IMAGE"
        
        # 从路径中提取文件扩展名
        suffix = Path(zip_path).suffix.lower()
        return _convert_image_data_to_uri(image_data, suffix)
        
    except Exception as e:
        print(f"转换ZIP图片失败 {zip_path}: {e}")
        return ""

def _convert_image_data_to_uri(image_data: bytes, suffix: str) -> str:
    """
    将图片数据转换为 data URI 格式
    
    Args:
        image_data: 图片的二进制数据
        suffix: 文件扩展名
    
    Returns:
        data URI 字符串
    """
    # 确定 MIME 类型
    if suffix == '.png':
        mime_type = 'image/png'
    elif suffix == '.webp':
        mime_type = 'image/webp'
    elif suffix in ['.jpg', '.jpeg']:
        mime_type = 'image/jpeg'
    elif suffix == '.gif':
        mime_type = 'image/gif'
    else:
        mime_type = 'image/png'  # 默认
    
    # 转换为 base64
    base64_data = base64.b64encode(image_data).decode('utf-8')
    
    # 检查转换后的大小
    if len(base64_data) > 4 * 1024 * 1024:  # 4MB限制
        print(f"Base64数据太大: {len(base64_data)} 字符")
        return ""
    
    # 构建 data URI
    return f"data:{mime_type};base64,{base64_data}"

async def render_guess_result(
    guessed_operator: Optional[Dict],
    comparison: Dict,
    attempts_left: int,
    mode: str = "大头"
) -> bytes:
    # 设置图片尺寸
    width = 450   # 调整为正方形比例 1:1
    height = 450   # 保持高度不变
    
    # 兔头模式和大头模式使用相同的立绘文件
    illustration_filename = guessed_operator.get("illustration", "")
    
    illustration_path = get_local_image_path(illustration_filename)
    illustration_uri = image_to_data_uri(illustration_path)
    
    # 检查图像是否缺失
    if illustration_uri == "MISSING_IMAGE":
        # 生成缺失图像的提示信息
        operator_name = guessed_operator.get("name", "未知干员")
        rarity = guessed_operator.get("rarity", "未知")
        career = guessed_operator.get("profession", "未知")
        
        missing_msg = f"⚠️ 立绘资源缺失\n"
        missing_msg += f"干员：{operator_name}\n"
        missing_msg += f"稀有度：{rarity}星\n"
        missing_msg += f"职业：{career}\n"
        missing_msg += f"💡 请使用 [arkstart 更新 立绘] 来下载立绘资源"
        
        # 将缺失图像信息传递给模板
        illustration_uri = f"data:text/plain;base64,{missing_msg}"
    
    # 移除了背景图功能
    
    # 根据模式选择模板
    template_name = "guess_rabbit.html" if mode == "兔头" else "guess.html"
    template = env.get_template(template_name)
    
    if mode == "兔头":
        # 兔头模式参数
        html = await template.render_async(
            operator_name=guessed_operator["name"],
            attempts_left=attempts_left,
            attack=guessed_operator.get("attack", "未知"),
            attack_comparison=comparison.get("attack", {}),
            defense=guessed_operator.get("defense", "未知"),
            defense_comparison=comparison.get("defense", {}),
            hp=guessed_operator.get("hp", "未知"),
            hp_comparison=comparison.get("hp", {}),
            res=guessed_operator.get("res", "未知"),
            res_comparison=comparison.get("res", {}),
            rarity=guessed_operator["rarity"],
            rarity_comparison=comparison["rarity"],
            gender=guessed_operator["gender"],
            gender_correct=comparison["gender"],
            interval=guessed_operator.get("interval", "未知"),
            interval_comparison=comparison.get("interval", {}),
            cost=guessed_operator.get("cost", "未知"),
            cost_comparison=comparison.get("cost", {}),
            tags=guessed_operator.get("tags", []),
            tags_comparison=comparison.get("tags", {}),
            all_correct=comparison.get("all_correct", False),
            illustration=illustration_uri,
            width=width,
            height=height
        )
    else:
        # 大头模式参数
        html = await template.render_async(
            operator_name=guessed_operator["name"],
            attempts_left=attempts_left,
            profession=guessed_operator.get("profession", "未知"),
            profession_correct=comparison.get("profession", False),
            subProfession=guessed_operator.get("subProfession", "未知"),
            subProfession_correct=comparison.get("subProfession", False),
            rarity=guessed_operator["rarity"],
            rarity_class=comparison["rarity"],
            origin=guessed_operator.get("origin", "未知"),
            origin_correct=comparison.get("origin", False),
            race=guessed_operator.get("race", "未知"),
            race_correct=comparison.get("race", False),
            gender=guessed_operator["gender"],
            gender_correct=comparison["gender"],
            position=guessed_operator.get("position", "未知"),
            position_correct=comparison.get("position", False),
            faction=guessed_operator.get("faction", "未知"),
            parent_faction=guessed_operator.get("parentFaction", ""),
            faction_comparison=comparison.get("faction", {}),
            tags=guessed_operator.get("tags", []),
            tags_comparison=comparison.get("tags", {}),
            illustration=illustration_uri,
            width=width,
            height=height
        )
    
    return await html_to_pic(html, viewport={"width": width, "height": height})

async def render_correct_answer(operator: Dict, mode: str = "大头") -> bytes:
    # 设置图片尺寸
    width = 450   # 调整为正方形比例 1:1
    height = 450   # 保持高度不变
    
    # 兔头模式和大头模式使用相同的立绘文件
    illustration_filename = operator.get("illustration", "")
    
    illustration_path = get_local_image_path(illustration_filename)
    illustration_uri = image_to_data_uri(illustration_path)
    
    # 检查图像是否缺失
    if illustration_uri == "MISSING_IMAGE":
        # 立绘缺失时抛出异常，不继续渲染
        operator_name = operator.get("name", "未知干员")
        rarity = operator.get("rarity", "未知")
        career = operator.get("profession", "未知")
        
        missing_msg = f"立绘资源缺失\n干员：{operator_name}\n稀有度：{rarity}星\n职业：{career}\n请使用 [arkstart 更新 立绘]来下载立绘资源"
        
        raise ValueError(missing_msg)
    
    # 移除了背景图功能
    
    # 根据模式选择模板
    template_name = "correct_rabbit.html" if mode == "兔头" else "correct.html"
    template = env.get_template(template_name)
    
    if mode == "兔头":
        # 兔头模式参数
        html = await template.render_async(
            name=operator.get("name", "未知干员"),
            attack=operator.get("attack", "未知"),
            defense=operator.get("defense", "未知"),
            hp=operator.get("hp", "未知"),
            res=operator.get("res", "未知"),
            rarity=operator.get("rarity", 1),
            gender=operator.get("gender", ""),
            interval=operator.get("interval", "未知"),
            cost=operator.get("cost", "未知"),
            tags=operator.get("tags", []),
            illustration=illustration_uri,
            width=width,
            height=height
        )
    else:
        # 大头模式参数
        html = await template.render_async(
            name=operator.get("name", "未知干员"),
            profession=operator.get("profession", "未知"),
            subProfession=operator.get("subProfession", "未知"),
            rarity=operator.get("rarity", 1),
            origin=operator.get("origin", "未知"),
            race=operator.get("race", "未知"),
            gender=operator.get("gender", ""),
            position=operator.get("position", "未知"),
            faction=operator.get("faction", "未知"),
            parent_faction=operator.get("parentFaction", ""),
            tags=operator.get("tags", []),
            illustration=illustration_uri,
            width=width,
            height=height
        )
    
    return await html_to_pic(html, viewport={"width": width, "height": height})


import random
import secrets
import json
import csv
import difflib
from pypinyin import lazy_pinyin
from pathlib import Path
def _safe_get_data_dir():
    try:
        from nonebot_plugin_localstore import get_plugin_data_dir as _get_dir  # type: ignore
        return _get_dir()
    except Exception:
        return Path.home() / ".local" / "share" / "nonebot2" / "nonebot_plugin_arkguesser"
from typing import Dict, List, Optional, Tuple
from datetime import datetime
from nonebot_plugin_uninfo import Uninfo
from .config import get_plugin_config
from .pool_manager import pool_manager
from .mode_manager import mode_manager
import zipfile

# 初始化随机数生成器，确保每次运行都有不同的随机性
def _init_random():
    """初始化随机数生成器"""
    # 使用当前时间和进程ID作为种子，确保随机性
    import time
    import os
    seed = int(time.time() * 1000000) + os.getpid()
    random.seed(seed)
    
    # 预热随机数生成器
    for _ in range(100):
        random.random()

# 在模块导入时初始化随机数
_init_random()

class OperatorGuesser:
    def __init__(self):
        self.games: Dict[str, Dict] = {}
        # 数据文件路径：使用 localstore 插件数据目录
        self.data_path = _safe_get_data_dir()
        
        # 立绘路径：使用 localstore 插件数据目录
        self.illustrations_path = self.data_path / "illustrations"
        
        # 初始化ZIP文件路径 - 兼容旧资源存档，仍放置于数据目录
        self.illustrations_zip_path = self.data_path / "images" / "illustrations.zip"
        self._zip_cache = {}  # 缓存ZIP文件内容
        
        # 检查数据文件是否存在
        if not self._check_data_files():
            # 数据文件不存在，初始化空数据
            self.operators = []
            self.career_map = {}
            self.camp_map = {}
            self.operator_names = []
            self.pinyin_operators = []
            self.max_attempts = get_plugin_config().arkguesser_max_attempts
            self._data_available = False
        else:
            # 数据文件存在，正常加载
            self.operators = self._load_data()
            self.career_map = self._load_career_map()
            self.camp_map = self._load_camp_map()
            self.max_attempts = get_plugin_config().arkguesser_max_attempts
            self.operator_names = [o["name"] for o in self.operators]  # 预加载干员名称列表
            self.pinyin_operators = [''.join(lazy_pinyin(operator)) for operator in self.operator_names]  # 预加载干员名称拼音列表
            self._data_available = True

    def _load_data(self) -> List[Dict]:
        """从CSV文件加载干员数据"""
        operators = []
        csv_path = self.data_path / "characters.csv"
        
        with open(csv_path, "r", encoding="utf-8-sig") as f:  # 使用utf-8-sig处理BOM
            reader = csv.DictReader(f)
            for idx, row in enumerate(reader, 1):
                # 适配新的字段结构
                operator_name = row["name"]  # CSV中的name字段是干员名称
                operator = {
                    "id": idx,
                    "name": operator_name,
                    "enName": row["id"],  # CSV中的id字段是英文ID
                    "profession": row["career"],  # 职业
                    "subProfession": row["subcareer"],  # 子职业
                    "rarity": int(row["rarity"]),  # 星级
                    "origin": row["birthplace"],  # 出身地
                    "race": row["race"],  # 种族
                    "gender": row["gender"],  # 性别
                    "parentFaction": row["camp"],  # 上级势力
                    "faction": row["subcamp"],  # 下级势力
                    "position": row["position"],  # 部署位置
                    "tags": [row.get("tag1", ""), row.get("tag2", ""), row.get("tag3", ""), row.get("tag4", "")],  # 标签
                    "illustration": self._get_illustration_path(operator_name, int(row["rarity"]), row["career"]),  # 立绘路径
                    # 数值属性
                    "attack": int(row.get("max_atk", 0)),  # 攻击
                    "defense": int(row.get("max_def", 0)),  # 防御
                    "hp": int(row.get("max_hp", 0)),  # 生命值上限
                    "res": int(row.get("max_magic_res", 0)),  # 法抗
                    "interval": self._parse_attack_speed(row.get("attack_speed", "0")),  # 攻击间隔
                    "cost": int(row.get("deploy_cost", 0))  # 初始费用
                }
                # 过滤空标签
                operator["tags"] = [tag for tag in operator["tags"] if tag.strip()]
                
                operators.append(operator)
        
        return operators
    
    def _check_data_files(self) -> bool:
        """检查必要的数据文件是否存在"""
        required_files = [
            "characters.csv",
            "career.json", 
            "camp.json"
        ]
        
        for filename in required_files:
            file_path = self.data_path / filename
            if not file_path.exists():
                print(f"数据文件缺失: {file_path}")
                return False
        
        return True
    
    def _parse_attack_speed(self, speed_str: str) -> float:
        """解析攻击间隔字符串，提取数值部分"""
        try:
            # 移除单位 's' 并转换为浮点数
            if speed_str and isinstance(speed_str, str):
                # 提取数字部分（可能包含小数点）
                import re
                match = re.search(r'(\d+\.?\d*)', speed_str)
                if match:
                    return float(match.group(1))
            return 0.0
        except (ValueError, TypeError):
            return 0.0
    
    def _load_career_map(self) -> Dict:
        """加载职业映射数据"""
        career_path = self.data_path / "career.json"
        with open(career_path, "r", encoding="utf-8-sig") as f:
            return json.load(f)
    
    def _load_camp_map(self) -> Dict:
        """加载势力映射数据"""
        camp_path = self.data_path / "camp.json"
        with open(camp_path, "r", encoding="utf-8-sig") as f:
            return json.load(f)
    
    def _get_illustration_path(self, name: str, rarity: int, career: str, mode: str = "大头") -> str:
        """获取干员立绘路径"""
        # 新的目录结构：稀有度X/职业/半身像/干员名_半身像_精英X.png
        rarity_folder = f"稀有度{rarity}"
        
        # 根据稀有度选择立绘类型
        # 稀有度4及以上使用精英2立绘，稀有度3及以下使用精英1立绘
        if rarity >= 4:
            illustration_type = "半身像_精英2"
        else:
            illustration_type = "半身像_精英1"
        
        # 构建新的文件路径
        # 首先尝试从新的目录结构获取
        new_path = f"{rarity_folder}/{career}/半身像/{name}_{illustration_type}.png"
        
        # 检查新路径是否存在
        if self._check_new_illustration_exists(new_path):
            return new_path
        
        # 如果新路径不存在，回退到旧的ZIP文件方式
        folder = str(rarity) if 1 <= rarity <= 6 else "6"
        webp_path = f"{folder}/{name}.webp"
        png_path = f"{folder}/{name}.png"
        
        # 检查ZIP文件中是否存在WebP文件
        if self._check_zip_file_exists(webp_path):
            return webp_path
        else:
            return png_path
    
    def check_illustration_availability(self, name: str, rarity: int, career: str) -> tuple[bool, str]:
        """
        检查干员立绘是否可用
        
        Returns:
            (是否可用, 提示信息)
        """
        # 检查新的目录结构
        rarity_folder = f"稀有度{rarity}"
        # 根据稀有度选择立绘类型
        if rarity >= 4:
            illustration_type = "半身像_精英2"
        else:
            illustration_type = "半身像_精英1"
        
        new_path = f"{rarity_folder}/{career}/半身像/{name}_{illustration_type}.png"
        if self._check_new_illustration_exists(new_path):
            return True, ""
        
        # 检查ZIP文件
        folder = str(rarity) if 1 <= rarity <= 6 else "6"
        webp_path = f"{folder}/{name}.webp"
        png_path = f"{folder}/{name}.png"
        
        if self._check_zip_file_exists(webp_path) or self._check_zip_file_exists(png_path):
            return True, ""
        
        # 立绘不可用，返回提示信息
        missing_msg = f"⚠️ 立绘资源缺失\n"
        missing_msg += f"干员：{name}\n"
        missing_msg += f"稀有度：{rarity}星\n"
        missing_msg += f"职业：{career}\n"
        missing_msg += f"💡 请使用 /arkstart 更新 来下载立绘资源"
        
        return False, missing_msg

    def _check_new_illustration_exists(self, file_path: str) -> bool:
        """检查新的立绘文件是否存在"""
        full_path = self.illustrations_path / file_path
        return full_path.exists()

    def _check_zip_file_exists(self, file_path: str) -> bool:
        """检查ZIP文件中是否存在指定文件"""
        if not self.illustrations_zip_path.exists():
            return False
            
        try:
            with zipfile.ZipFile(self.illustrations_zip_path, 'r') as zip_file:
                return file_path in zip_file.namelist()
        except Exception as e:
            print(f"检查ZIP文件失败: {e}")
            return False

    def _get_zip_file_content(self, file_path: str) -> Optional[bytes]:
        """从ZIP文件中获取文件内容"""
        if not self.illustrations_zip_path.exists():
            return None
            
        try:
            with zipfile.ZipFile(self.illustrations_zip_path, 'r') as zip_file:
                if file_path in zip_file.namelist():
                    return zip_file.read(file_path)
        except Exception as e:
            print(f"读取ZIP文件失败: {e}")
        return None

    def get_session_id(self, uninfo) -> str:
        return f"{uninfo.scope}_{uninfo.self_id}_{uninfo.scene_path}"

    def is_data_available(self) -> bool:
        """检查数据是否可用"""
        return getattr(self, '_data_available', False)
    
    def reload_data(self) -> bool:
        """重新加载数据文件"""
        try:
            # 重新检查数据文件
            if self._check_data_files():
                # 数据文件存在，重新加载
                self.operators = self._load_data()
                self.career_map = self._load_career_map()
                self.camp_map = self._load_camp_map()
                self.operator_names = [o["name"] for o in self.operators]
                self.pinyin_operators = [''.join(lazy_pinyin(operator)) for operator in self.operator_names]
                self._data_available = True
                print("✅ 数据重新加载成功")
                return True
            else:
                # 数据文件仍然缺失
                self.operators = []
                self.career_map = {}
                self.camp_map = {}
                self.operator_names = []
                self.pinyin_operators = []
                self._data_available = False
                print("❌ 数据文件仍然缺失")
                return False
        except Exception as e:
            print(f"❌ 重新加载数据失败: {e}")
            self._data_available = False
            return False

    def get_game(self, uninfo: Uninfo) -> Optional[Dict]:
        return self.games.get(self.get_session_id(uninfo))

    def start_new_game(self, uninfo: Uninfo) -> Dict:
        """开始新游戏"""
        # 检查数据是否可用
        if not self.is_data_available():
            raise ValueError("数据文件不可用，请先使用 [arkstart 更新] 来下载干员数据")
        
        session_id = self.get_session_id(uninfo)
        
        # 获取当前题库设置
        from .pool_manager import pool_manager
        user_id = str(uninfo.user.id) if uninfo.user else None
        group_id = str(uninfo.group.id) if uninfo.group else None
        pool_info = pool_manager.get_pool_info(user_id, group_id)
        allowed_rarities = pool_info["rarity_list"]
        
        # 获取当前游戏模式设置
        from .mode_manager import mode_manager
        mode_info = mode_manager.get_mode_info(user_id, group_id)
        current_mode = mode_info["mode"]
        
        # 获取连战模式设置
        from .continuous_manager import ContinuousManager
        continuous_manager = ContinuousManager()
        continuous_enabled = continuous_manager.get_continuous_mode(user_id, group_id)
        
        # 从题库中随机选择干员
        available_operators = [o for o in self.operators if o["rarity"] in allowed_rarities]
        if not available_operators:
            raise ValueError("当前题库中没有可用干员")
        
        # 使用更安全的随机选择方法
        if len(available_operators) == 1:
            selected_operator = available_operators[0]
        else:
            # 使用secrets模块进行更安全的随机选择
            selected_index = secrets.randbelow(len(available_operators))
            selected_operator = available_operators[selected_index]
        
        # 检查选中干员的立绘是否可用
        operator_name = selected_operator["name"]
        operator_rarity = selected_operator["rarity"]
        operator_career = selected_operator["profession"]
        
        is_illustration_available, missing_msg = self.check_illustration_availability(
            operator_name, operator_rarity, operator_career
        )
        
        if not is_illustration_available:
            # 立绘不可用，抛出异常
            raise ValueError(f"无法开始游戏：{missing_msg}")
        
        # 创建游戏数据
        game_data = {
            "operator": selected_operator,
            "guesses": [],
            "start_time": datetime.now(),
            "allowed_rarities": allowed_rarities,
            "current_mode": current_mode,
            "continuous_mode": continuous_enabled,  # 保存连战模式状态
            "continuous_count": 0,  # 连战次数计数
            "session_id": session_id,
            "user_id": user_id,  # 保存用户ID用于后续连战模式检查
            "group_id": group_id  # 保存群组ID用于后续连战模式检查
        }
        
        self.games[session_id] = game_data
        return game_data

    def guess(self, uninfo: Uninfo, name: str) -> Tuple[bool, Optional[Dict], Dict]:
        # 检查数据是否可用
        if not self.is_data_available():
            raise ValueError("数据文件不可用，请先使用 [arkstart 更新] 来下载干员数据")
        
        game = self.get_game(uninfo)
        if not game or len(game["guesses"]) >= self.max_attempts:
            raise ValueError("游戏已结束")

        guessed = next((o for o in self.operators if o["name"] == name), None)
        if not guessed:
            return False, None, {}

        game["guesses"].append(guessed)
        current = game["operator"]

        # 势力比较逻辑
        faction_comparison = self._compare_factions(
            guessed.get("parentFaction", ""),
            guessed.get("faction", ""),
            current.get("parentFaction", ""),
            current.get("faction", "")
        )
        
        # 标签比较逻辑
        tags_comparison = self._compare_tags(
            guessed.get("tags", []),
            current.get("tags", [])
        )
        
        # 获取当前游戏模式
        current_mode = game.get("current_mode", "大头")
        
        if current_mode == "兔头":
            # 兔头模式：比较游戏数值属性
            comparison = {
                "attack": self._compare_numeric_value(guessed.get("attack", 0), current.get("attack", 0)),
                "defense": self._compare_numeric_value(guessed.get("defense", 0), current.get("defense", 0)),
                "hp": self._compare_numeric_value(guessed.get("hp", 0), current.get("hp", 0)),
                "res": self._compare_numeric_value(guessed.get("res", 0), current.get("res", 0)),
                "rarity": self._compare_rarity(guessed["rarity"], current["rarity"]),
                "gender": guessed["gender"] == current["gender"],
                "interval": self._compare_numeric_value(guessed.get("interval", 0), current.get("interval", 0)),
                "cost": self._compare_numeric_value(guessed.get("cost", 0), current.get("cost", 0)),
                "tags": tags_comparison
            }
            
            # 检查是否所有属性都正确
            all_correct = (
                comparison["attack"]["correct"] and
                comparison["defense"]["correct"] and
                comparison["hp"]["correct"] and
                comparison["res"]["correct"] and
                comparison["rarity"]["correct"] and
                comparison["gender"] and
                comparison["interval"]["correct"] and
                comparison["cost"]["correct"]
            )
            comparison["all_correct"] = all_correct
        else:
            # 大头模式：比较原有属性
            comparison = {
                "profession": guessed["profession"] == current["profession"],
                "subProfession": guessed["subProfession"] == current["subProfession"],
                "rarity": "higher" if guessed["rarity"] > current["rarity"]
                else "lower" if guessed["rarity"] < current["rarity"]
                else "same",
                "origin": guessed["origin"] == current["origin"],
                "race": guessed["race"] == current["race"],
                "gender": guessed["gender"] == current["gender"],
                "position": guessed["position"] == current["position"],
                "faction": faction_comparison,
                "tags": tags_comparison
            }
        return guessed["name"] == current["name"], guessed, comparison

    def find_similar_operators(self, name: str, n: int = 3) -> List[str]:
        # 检查数据是否可用
        if not self.is_data_available():
            return []
        
        # 使用difflib找到相似的干员名称
        difflib_matches = difflib.get_close_matches(
            name,
            self.operator_names,
            n=n,
            cutoff=0.6  # 相似度阈值（0-1之间）
        )
        # 通过拼音精确匹配读音一样的干员名称
        name_pinyin = ''.join(lazy_pinyin(name))  # 转换输入名称为拼音
        pinyin_matches = [self.operator_names[i] for i, pinyin in enumerate(self.pinyin_operators) if
                          pinyin == name_pinyin]

        all_matches = list(dict.fromkeys(pinyin_matches + difflib_matches))
        return all_matches

    def _compare_tags(self, guessed_tags: List[str], target_tags: List[str]) -> Dict:
        """
        比较标签信息，支持乱序匹配
        
        Args:
            guessed_tags: 猜测干员的标签列表
            target_tags: 目标干员的标签列表
        
        Returns:
            包含匹配结果的字典，包括状态信息
        """
        # 清理空标签
        guessed_tags = [tag.strip() for tag in guessed_tags if tag.strip()]
        target_tags = [tag.strip() for tag in target_tags if tag.strip()]
        
        # 找出匹配的标签
        matched_tags = []
        for tag in guessed_tags:
            if tag in target_tags:
                matched_tags.append(tag)
        
        # 确定匹配状态
        match_count = len(matched_tags)
        total_guessed = len(guessed_tags)
        total_target = len(target_tags)
        
        if match_count == total_guessed and match_count == total_target:
            # 完全匹配：所有标签都一致
            status = "exact_match"
        elif match_count > 0:
            # 部分匹配：有些标签匹配
            status = "partial_match"
        else:
            # 无匹配：没有标签匹配
            status = "no_match"
        
        return {
            "matched_tags": matched_tags,  # 匹配的标签列表
            "total_guessed": total_guessed,  # 猜测干员的标签总数
            "total_target": total_target,  # 目标干员的标签总数
            "match_count": match_count,  # 匹配的标签数量
            "status": status  # 匹配状态
        }

    def _compare_numeric_value(self, guessed_value: int, target_value: int) -> Dict:
        """
        比较数值属性，返回详细的比较信息
        
        Args:
            guessed_value: 猜测的数值
            target_value: 目标数值
        
        Returns:
            包含比较结果的字典
        """
        if target_value == 0:
            # 避免除零错误
            return {
                "correct": guessed_value == target_value,
                "direction": "same" if guessed_value == target_value else "unknown",
                "percentage_diff": 0 if guessed_value == target_value else float('inf'),
                "within_20_percent": guessed_value == target_value
            }
        
        # 计算百分比差异
        percentage_diff = abs(guessed_value - target_value) / target_value * 100
        
        # 判断是否在20%范围内
        within_20_percent = percentage_diff <= 20
        
        # 确定差距方向
        if guessed_value == target_value:
            direction = "same"
        elif guessed_value < target_value:
            direction = "up"  # 答案大于猜测，显示↑
        else:
            direction = "down"  # 答案小于猜测，显示↓
        
        return {
            "correct": guessed_value == target_value,
            "direction": direction,
            "percentage_diff": percentage_diff,
            "within_20_percent": within_20_percent
        }

    def _compare_rarity(self, guessed_rarity: int, target_rarity: int) -> Dict:
        """
        比较星级，返回详细的比较信息
        
        Args:
            guessed_rarity: 猜测的星级
            target_rarity: 目标星级
        
        Returns:
            包含比较结果的字典
        """
        if guessed_rarity == target_rarity:
            return {
                "correct": True,
                "direction": "same",
                "percentage_diff": 0,
                "within_20_percent": True
            }
        elif guessed_rarity > target_rarity:
            return {
                "correct": False,
                "direction": "down",
                "percentage_diff": (guessed_rarity - target_rarity) / target_rarity * 100,
                "within_20_percent": False
            }
        else:
            return {
                "correct": False,
                "direction": "up",
                "percentage_diff": (target_rarity - guessed_rarity) / target_rarity * 100,
                "within_20_percent": False
            }

    def _compare_factions(self, guess_parent: str, guess_faction: str, 
                         target_parent: str, target_faction: str) -> Dict:
        """
        比较势力信息，支持分层势力系统
        
        Args:
            guess_parent: 猜测的上级势力
            guess_faction: 猜测的下级势力
            target_parent: 目标的上级势力
            target_faction: 目标的下级势力
        
        Returns:
            包含比较结果的字典
        """
        # 完全匹配
        if (guess_parent == target_parent and guess_faction == target_faction):
            return {
                "status": "exact_match",
                "parent_match": True,
                "faction_match": True,
                "display_parent": guess_parent,
                "display_faction": guess_faction
            }
        
        # 上级势力匹配
        elif guess_parent == target_parent:
            return {
                "status": "parent_match",
                "parent_match": True,
                "faction_match": False,
                "display_parent": guess_parent,
                "display_faction": guess_faction
            }
        
        # 完全不匹配
        else:
            return {
                "status": "no_match",
                "parent_match": False,
                "faction_match": False,
                "display_parent": guess_parent,
                "display_faction": guess_faction
            }

    def end_game(self, uninfo: Uninfo):
        try:
            self.games.pop(self.get_session_id(uninfo))
        except (AttributeError, KeyError):
            pass
    
    def update_continuous_count(self, uninfo: Uninfo, increment: bool = True) -> int:
        """更新连战计数"""
        session_id = self.get_session_id(uninfo)
        if session_id in self.games:
            if increment:
                self.games[session_id]["continuous_count"] += 1
            return self.games[session_id]["continuous_count"]
        return 0
    
    def get_continuous_count(self, uninfo: Uninfo) -> int:
        """获取当前连战计数"""
        session_id = self.get_session_id(uninfo)
        if session_id in self.games:
            return self.games[session_id].get("continuous_count", 0)
        return 0
    
    def is_continuous_mode(self, uninfo: Uninfo) -> bool:
        """检查当前是否处于连战模式"""
        session_id = self.get_session_id(uninfo)
        if session_id in self.games:
            return self.games[session_id].get("continuous_mode", False)
        return False
    
    def get_random_quality_info(self) -> Dict:
        """获取随机数质量信息，用于调试和验证"""
        import time
        
        # 生成一些随机数样本
        random_samples = [random.random() for _ in range(100)]
        
        # 计算随机数的分布情况
        min_val = min(random_samples)
        max_val = max(random_samples)
        avg_val = sum(random_samples) / len(random_samples)
        
        # 检查是否有重复值（理论上不应该有）
        unique_count = len(set(random_samples))
        
        return {
            "seed_time": time.time(),
            "samples_count": len(random_samples),
            "unique_count": unique_count,
            "min_value": min_val,
            "max_value": max_val,
            "average_value": avg_val,
            "randomness_score": unique_count / len(random_samples),  # 越接近1越好
            "using_secrets": True
        }


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
明日方舟干员立绘自动下载程序 V2
基于 nonebot-plugin-arkguesser-main 的干员数据
使用 PRTS Wiki API 获取干员立绘图片
增强版本，支持更多配置选项和功能
"""

import asyncio
import csv
import json
import re
import time
import hashlib
from pathlib import Path
from typing import Dict, List, Optional, Union, Tuple
from urllib.parse import quote
from dataclasses import dataclass

import httpx
from PIL import Image
from nonebot import logger

# 兼容相对/绝对导入，支持脚本独立运行
try:
    from .illustration_config import *  # type: ignore
except Exception:
    from illustration_config import *  # type: ignore


@dataclass
class Operator:
    """干员信息数据类"""
    charid: str
    name: str
    original_name: str
    rarity: str
    career: str
    subcareer: str = ""
    camp: str = ""
    race: str = ""


@dataclass
class Illustration:
    """立绘信息数据类"""
    type: str
    filename: str
    charid: str
    level: Optional[int] = None
    url: Optional[str] = None
    file_size: Optional[int] = None
    md5_hash: Optional[str] = None


class ArknightsIllustrationDownloaderV2:
    """明日方舟干员立绘下载器 V2"""
    
    def __init__(self, output_dir: Optional[str] = None, config: Optional[Dict] = None):
        self.output_dir = Path(output_dir) if output_dir else OUTPUT_DIR
        self.output_dir.mkdir(exist_ok=True)
        
        # 使用配置文件
        self.config = config or {}
        self.download_config = {**DOWNLOAD_CONFIG, **self.config.get('download', {})}
        self.prts_config = {**PRTS_CONFIG, **self.config.get('prts', {})}
        self.illustration_types = {**ILLUSTRATION_TYPES, **self.config.get('illustration_types', {})}
        
        # 干员数据文件路径
        self.characters_csv_path = CHARACTERS_CSV_PATH
        
        # 下载统计
        self.stats = {
            "total_operators": 0,
            "successful_downloads": 0,
            "failed_downloads": 0,
            "skipped_downloads": 0,
            "total_file_size": 0,
            "start_time": None,
            "end_time": None
        }
        
        # 失败列表
        self.failed_downloads = []
        
        # 不修改用户日志配置
    
    def _setup_logging(self):
        """兼容方法：不再添加自定义日志处理器"""
        return
    
    def load_operators_from_csv(self) -> List[Operator]:
        """从CSV文件加载干员数据"""
        logger.info("正在从CSV文件加载干员数据...")
        
        operators = []
        
        try:
            with open(self.characters_csv_path, 'r', encoding='utf-8-sig') as f:  # 使用utf-8-sig处理BOM
                reader = csv.DictReader(f)
                for row in reader:
                    # 清理数据，去除多余的字符
                    char_id = row['id'].strip()
                    name = row['name'].strip()
                    rarity = row['rarity'].strip()
                    career = row['career'].strip()
                    subcareer = row.get('subcareer', '').strip()
                    camp = row.get('camp', '').strip()
                    race = row.get('race', '').strip()
                    
                    # 跳过无效数据（包括表头行）
                    if not char_id or not name or char_id == 'id' or name == 'name':
                        continue
                    
                    # 应用过滤条件
                    if not self._should_include_operator(char_id, rarity, career):
                        continue
                    
                    # 清理名称中的特殊字符
                    clean_name = re.sub(r'[\(\)（）]', '', name)
                    clean_name = clean_name.strip()
                    
                    operator = Operator(
                        charid=char_id,
                        name=clean_name,
                        original_name=name,
                        rarity=rarity,
                        career=career,
                        subcareer=subcareer,
                        camp=camp,
                        race=race
                    )
                    
                    operators.append(operator)
            
            logger.success(f"成功加载 {len(operators)} 个干员信息")
            self.stats["total_operators"] = len(operators)
            return operators
            
        except Exception as e:
            logger.error(f"加载CSV文件失败: {e}")
            return []
    
    def _should_include_operator(self, char_id: str, rarity: str, career: str) -> bool:
        """判断是否应该包含该干员"""
        filter_config = FILTER_CONFIG.copy()
        filter_config.update(self.config.get('filter', {}))
        
        # 检查是否在排除列表中
        if char_id in filter_config.get('exclude_operators', []):
            return False
        
        # 检查稀有度范围
        try:
            rarity_num = int(rarity)
            if not (filter_config['min_rarity'] <= rarity_num <= filter_config['max_rarity']):
                return False
        except (ValueError, TypeError):
            pass
        
        # 检查职业
        if career and filter_config.get('include_careers'):
            if career not in filter_config['include_careers']:
                return False
        
        return True
    
    def _get_output_path(self, operator: Operator, illustration: Illustration) -> Path:
        """获取输出文件路径"""
        output_config = OUTPUT_CONFIG.copy()
        output_config.update(self.config.get('output', {}))
        
        if not output_config['create_subdirectories']:
            return self.output_dir / f"{operator.name}_{illustration.type}_{illustration.level or '1'}.png"
        
        # 创建子目录结构
        subdirs = []
        
        if output_config['subdirectory_structure'].get('by_rarity'):
            subdirs.append(f"稀有度{operator.rarity}")
        
        if output_config['subdirectory_structure'].get('by_career'):
            subdirs.append(operator.career)
        
        if output_config['subdirectory_structure'].get('by_type'):
            subdirs.append(illustration.type)
        
        output_path = self.output_dir
        for subdir in subdirs:
            output_path = output_path / subdir
            output_path.mkdir(exist_ok=True)
        
        # 构建文件名
        naming_config = NAMING_CONFIG.copy()
        naming_config.update(self.config.get('naming', {}))
        
        filename_parts = [operator.name]
        
        if naming_config['include_rarity']:
            filename_parts.append(f"稀有度{operator.rarity}")
        
        if naming_config['include_career']:
            filename_parts.append(operator.career)
        
        filename_parts.append(illustration.type)
        
        if naming_config['include_level'] and illustration.level:
            filename_parts.append(f"精英{illustration.level}")
        
        filename = naming_config['separator'].join(filename_parts) + ".png"
        
        return output_path / filename
    
    async def get_operator_illustrations(self, client: httpx.AsyncClient, operator: Operator) -> List[Illustration]:
        """获取干员的立绘信息"""
        logger.debug(f"正在获取 {operator.name}({operator.charid}) 的立绘信息...")
        
        illustrations = []
        
        # 直接使用默认的立绘路径，跳过页面解析
        try:
            # 标准半身像路径
            for level in [1, 2]:  # 精英化等级
                illustrations.append(Illustration(
                    type="半身像",
                    filename=f"{operator.charid}_{level}.png",
                    charid=operator.charid,
                    level=level
                ))
            
            # 头像路径
            illustrations.append(Illustration(
                type="头像", 
                filename=f"{operator.charid}.png",
                charid=operator.charid
            ))
            
            logger.debug(f"找到 {len(illustrations)} 个立绘文件")
            return illustrations
            
        except Exception as e:
            logger.error(f"获取 {operator.name} 立绘信息失败: {e}")
            return []
    
    async def download_illustration(self, client: httpx.AsyncClient, operator: Operator, illustration: Illustration) -> bool:
        """下载立绘图片"""
        try:
            # 构建下载URL
            if illustration.type == "半身像":
                level = illustration.level or 1
                url = f"{self.prts_config['assets_url']}/char_portrait/{operator.charid}_{level}.png"
                illustration.url = url
            elif illustration.type == "头像":
                url = f"{self.prts_config['assets_url']}/char_avatar/{operator.charid}.png"
                illustration.url = url
            else:
                # 如果有自定义文件名，尝试直接下载
                url = f"{self.prts_config['assets_url']}/char_portrait/{illustration.filename}"
                illustration.url = url
            
            logger.debug(f"正在下载: {url}")
            
            # 添加请求头
            headers = {
                "User-Agent": self.prts_config['user_agent']
            }
            
            response = await client.get(url, headers=headers, timeout=20.0)
            if response.status_code != 200:
                logger.warning(f"下载失败 {operator.name}: HTTP {response.status_code}")
                return False
            
            # 获取输出路径
            output_path = self._get_output_path(operator, illustration)
            
            # 保存图片
            with open(output_path, "wb") as f:
                f.write(response.content)
            
            # 更新立绘信息
            illustration.file_size = len(response.content)
            illustration.md5_hash = hashlib.md5(response.content).hexdigest()
            
            # 验证图片
            if not self._validate_image(output_path, illustration):
                output_path.unlink(missing_ok=True)
                return False
            
            logger.success(f"成功下载: {output_path.name}")
            return True
                
        except Exception as e:
            logger.error(f"下载立绘失败 {operator.name}: {e}")
            return False
    
    def _validate_image(self, file_path: Path, illustration: Illustration) -> bool:
        """验证图片文件"""
        validation_config = VALIDATION_CONFIG.copy()
        validation_config.update(self.config.get('validation', {}))
        
        try:
            # 检查文件大小
            file_size = file_path.stat().st_size
            if file_size < validation_config['min_file_size']:
                logger.warning(f"文件过小: {file_path.name} ({file_size} bytes)")
                return False
            
            if file_size > validation_config['max_file_size']:
                logger.warning(f"文件过大: {file_path.name} ({file_size} bytes)")
                return False
            
            # 验证图片完整性
            if validation_config['verify_image_integrity']:
                with Image.open(file_path) as img:
                    img.verify()
                    
                    # 检查图片格式
                    if img.format.lower() not in validation_config['allowed_formats']:
                        logger.warning(f"不支持的图片格式: {file_path.name} ({img.format})")
                        return False
            
            return True
            
        except Exception as e:
            logger.error(f"图片验证失败 {file_path.name}: {e}")
            return False
    
    async def download_operator_illustrations(self, client: httpx.AsyncClient, operator: Operator) -> Dict:
        """下载单个干员的所有立绘"""
        logger.info(f"正在处理干员: {operator.name} ({operator.charid})")
        
        # 获取立绘信息
        illustrations = await self.get_operator_illustrations(client, operator)
        
        if not illustrations:
            logger.warning(f"{operator.name}: 没有找到立绘信息")
            return {"name": operator.name, "success": 0, "total": 0, "skipped": True}
        
        # 下载立绘
        success_count = 0
        for illustration in illustrations:
            if await self.download_illustration(client, operator, illustration):
                success_count += 1
                self.stats["successful_downloads"] += 1
                if illustration.file_size:
                    self.stats["total_file_size"] += illustration.file_size
            else:
                self.stats["failed_downloads"] += 1
                self.failed_downloads.append({
                    "operator": operator.name,
                    "charid": operator.charid,
                    "illustration": illustration.type,
                    "error": "下载失败"
                })
        
        if success_count > 0:
            logger.info(f"{operator.name}: 成功下载 {success_count}/{len(illustrations)} 个立绘")
        else:
            logger.warning(f"{operator.name}: 没有成功下载任何立绘")
        
        return {"name": operator.name, "success": success_count, "total": len(illustrations), "skipped": False}
    
    async def download_all_illustrations(self, operators: List[Operator]):
        """下载所有干员的立绘"""
        logger.info(f"开始下载 {len(operators)} 个干员的立绘...")
        
        semaphore = asyncio.Semaphore(self.download_config['max_concurrent'])
        
        async def download_operator(operator: Operator):
            async with semaphore:
                return await self.download_operator_illustrations(client, operator)
        
        # 网络配置
        network_config = NETWORK_CONFIG.copy()
        network_config.update(self.config.get('network', {}))
        
        client_config = {
            "verify": network_config['verify_ssl'],
            "follow_redirects": network_config['follow_redirects'],
            "timeout": self.download_config['timeout']
        }
        
        if network_config['use_proxy'] and network_config['proxy']:
            client_config['proxies'] = network_config['proxy']
        
        async with httpx.AsyncClient(**client_config) as client:
            tasks = [download_operator(op) for op in operators]
            results = await asyncio.gather(*tasks, return_exceptions=True)
            
            # 处理结果
            for result in results:
                if isinstance(result, Exception):
                    logger.error(f"下载过程中出现异常: {result}")
                elif result.get("skipped"):
                    self.stats["skipped_downloads"] += 1
        
        logger.success("所有立绘下载完成！")
    
    def save_download_stats(self):
        """保存下载统计信息"""
        stats_file = self.output_dir / "download_stats.json"
        
        stats_data = {
            "download_time": {
                "start": self.stats["start_time"],
                "end": self.stats["end_time"],
                "duration": self.stats["end_time"] - self.stats["start_time"] if self.stats["start_time"] and self.stats["end_time"] else None
            },
            "statistics": self.stats,
            "failed_downloads": self.failed_downloads,
            "output_directory": str(self.output_dir.absolute()),
            "configuration": {
                "download": self.download_config,
                "filter": FILTER_CONFIG,
                "output": OUTPUT_CONFIG
            }
        }
        
        with open(stats_file, "w", encoding="utf-8") as f:
            json.dump(stats_data, f, ensure_ascii=False, indent=2)
        
        logger.info(f"下载统计已保存到: {stats_file}")
    
    def print_summary(self):
        """打印下载总结"""
        logger.info("=" * 60)
        logger.info("下载完成总结")
        logger.info("=" * 60)
        logger.info(f"总干员数: {self.stats['total_operators']}")
        logger.info(f"成功下载: {self.stats['successful_downloads']}")
        logger.info(f"下载失败: {self.stats['failed_downloads']}")
        logger.info(f"跳过下载: {self.stats['skipped_downloads']}")
        logger.info(f"总文件大小: {self.stats['total_file_size'] / (1024*1024):.2f} MB")
        
        if self.stats['start_time'] and self.stats['end_time']:
            duration = self.stats['end_time'] - self.stats['start_time']
            logger.info(f"总耗时: {duration}")
        
        logger.info(f"输出目录: {self.output_dir.absolute()}")
        
        if self.failed_downloads:
            logger.info(f"失败列表已保存到统计文件中")
        
        logger.info("=" * 60)
    
    async def run(self):
        """运行下载器"""
        logger.info("明日方舟干员立绘下载器 V2 启动")
        
        self.stats["start_time"] = time.time()
        
        # 加载干员数据
        operators = self.load_operators_from_csv()
        
        if not operators:
            logger.error("无法加载干员数据，程序退出")
            return
        
        # 保存干员信息到JSON文件
        if OUTPUT_CONFIG['save_metadata']:
            info_file = self.output_dir / "operators_info.json"
            operators_data = [
                {
                    "charid": op.charid,
                    "name": op.name,
                    "original_name": op.original_name,
                    "rarity": op.rarity,
                    "career": op.career,
                    "subcareer": op.subcareer,
                    "camp": op.camp,
                    "race": op.race
                }
                for op in operators
            ]
            
            with open(info_file, "w", encoding="utf-8") as f:
                json.dump(operators_data, f, ensure_ascii=False, indent=2)
            logger.info(f"干员信息已保存到: {info_file}")
        
        # 下载立绘
        await self.download_all_illustrations(operators)
        
        self.stats["end_time"] = time.time()
        
        # 保存统计信息
        self.save_download_stats()
        
        # 打印总结
        self.print_summary()


async def main():
    """主函数"""
    # 自定义配置示例 - 下载所有职业和星级的干员立绘
    custom_config = {
        'download': {
            'max_concurrent': 5,  # 增加并发数
            'timeout': 30,         # 减少超时时间
        },
        'filter': {
            'min_rarity': 1,       # 下载1星以上干员（包含所有星级）
            'max_rarity': 6,       # 下载6星干员
            'include_careers': [   # 包含所有职业
                "先锋", "近卫", "重装", "狙击", "术师", "医疗", "辅助", "特种"
            ]
        },
        'output': {
            'create_subdirectories': True,
            'subdirectory_structure': {
                'by_rarity': True,
                'by_career': True,
                'by_type': True
            }
        }
    }
    
    downloader = ArknightsIllustrationDownloaderV2(config=custom_config)
    await downloader.run()


if __name__ == "__main__":
    asyncio.run(main())



from nonebot import on_message, get_driver, require
from nonebot.plugin import PluginMetadata, inherit_supported_adapters
from nonebot.adapters import Event
from nonebot.matcher import Matcher
from nonebot.rule import Rule
from nonebot.exception import FinishedException
require("nonebot_plugin_alconna")
require("nonebot_plugin_uninfo")
require("nonebot_plugin_htmlrender")
require("nonebot_plugin_localstore")

from nonebot_plugin_uninfo import Uninfo
from nonebot_plugin_alconna import UniMessage, Image, on_alconna, Args
from arclet.alconna import Alconna, Option
from .config import ArkGuesserConfig

from .game import OperatorGuesser
from .render import render_guess_result, render_correct_answer
from .pool_manager import pool_manager
from .mode_manager import mode_manager
from .continuous_manager import ContinuousManager

# 创建管理器实例
pool_manager = pool_manager
mode_manager = mode_manager
continuous_manager = ContinuousManager()

# 导出插件元数据，确保 NoneBot 能正确识别
__all__ = ["__plugin_meta__"]

__plugin_meta__ = PluginMetadata(
    name="明日方舟猜干员游戏",
    description="明日方舟猜干员游戏 - 支持多种游戏模式和题库设置",
    usage="""🎮 游戏指令:
arkstart - 开始游戏
结束 - 结束游戏
直接输入干员名即可开始猜测

📚 题库设置:
/arkstart 题库 6 - 设置题库为6星干员
/arkstart 题库 4-6 - 设置题库为4-6星干员
/arkstart 题库 查看 - 查看当前题库设置
/arkstart 题库 重置 - 重置为默认设置

🎭 模式设置:
/arkstart 模式 大头 - 设置为大头模式
/arkstart 模式 兔头 - 设置为兔头模式
/arkstart 模式 查看 - 查看当前模式设置
/arkstart 模式 重置 - 重置为默认模式

🔄 连战模式设置:
/arkstart 连战 开启 - 开启连战模式
/arkstart 连战 关闭 - 关闭连战模式
/arkstart 连战 查看 - 查看当前连战模式设置
/arkstart 连战 重置 - 重置为默认连战模式设置""",
    type="application",
    homepage="https://github.com/lizhiqi233-rgb/nonebot-plugin-arkguesser",
    config=ArkGuesserConfig,
    supported_adapters=inherit_supported_adapters("nonebot_plugin_alconna"),
)

# 延迟创建游戏实例，避免在导入时调用配置
_game_instance = None

def get_game_instance():
    """获取游戏实例，延迟初始化"""
    global _game_instance
    if _game_instance is None:
        _game_instance = OperatorGuesser()
    return _game_instance

driver = get_driver()

def is_playing() -> Rule:
    async def _checker(uninfo: Uninfo) -> bool:
        return bool(get_game_instance().get_game(uninfo))
    return Rule(_checker)

# 简化 Alconna 结构，使用 Option 而不是 Subcommand
start_cmd = on_alconna(
    Alconna(
        "arkstart",
        Option("题库", Args["range_str;?", str]),
        Option("模式", Args["mode;?", str]),
        Option("连战", Args["action;?", str]),
        Option("更新", Args["update_type;?", str])
    ),
    aliases={"明日方舟开始"}
)

guess_matcher = on_message(rule=is_playing(), priority=15, block=False)

@start_cmd.handle()
async def handle_start(uninfo: Uninfo, matcher: Matcher, event: Event):
    try:
        # 获取原始消息文本
        message_text = str(event.get_message()) if hasattr(event, 'get_message') else ""
        
        # 检查是否是设置子命令
        if "题库" in message_text:
            await handle_pool_settings(uninfo, matcher, message_text)
            return
        
        if "模式" in message_text:
            await handle_mode_settings(uninfo, matcher, message_text)
            return
        
        if "连战" in message_text:
            await handle_continuous_settings(uninfo, matcher, message_text)
            return
        
        if "更新" in message_text:
            await handle_update_resources(uninfo, matcher, message_text)
            return
        
        # 处理开始游戏
        if get_game_instance().get_game(uninfo):
            await matcher.send("🎮 游戏已在进行中！\n💬 请继续猜测或输入「结束」来结束游戏")
            return
        
        game_data = get_game_instance().start_new_game(uninfo)
        
        # 获取连战模式设置
        user_id = str(uninfo.user.id) if uninfo.user else None
        group_id = str(uninfo.group.id) if uninfo.group else None
        continuous_enabled = continuous_manager.get_continuous_mode(user_id, group_id)
        
        # 设置连战模式状态
        game_data["continuous_mode"] = continuous_enabled
        
        # 显示游戏开始信息，包含当前题库范围和模式
        allowed_rarities = game_data.get("allowed_rarities", [6])
        current_mode = game_data.get("current_mode", "大头")
        
        if len(allowed_rarities) == 1:
            range_display = f"{allowed_rarities[0]}星"
        else:
            range_display = f"{min(allowed_rarities)}-{max(allowed_rarities)}星"
        
        mode_description = mode_manager._get_mode_description(current_mode)
        
        # 构建简洁的游戏开始提示
        start_msg = f"🎮 游戏开始！\n"
        start_msg += f"📚 {range_display} | 🎭 {current_mode}"
        
        # 添加连战模式状态
        if continuous_enabled:
            start_msg += f" | 🔄 连战"
        
        start_msg += f"\n🎯 {get_game_instance().max_attempts}次机会 | 💬 直接输入干员名"
        
        await matcher.send(start_msg)
    
    except FinishedException:
        # FinishedException 是正常的流程控制，直接返回
        return
    except Exception as e:
        # 添加错误处理
        import traceback
        error_msg = f"处理命令时出错: {str(e)}\n{traceback.format_exc()}"
        await matcher.send(f"❌ 插件运行出错，请检查日志: {str(e)}")

async def handle_continuous_settings(uninfo: Uninfo, matcher: Matcher, message_text: str):
    """统一的连战模式设置处理"""
    try:
        user_id = str(uninfo.user.id) if uninfo.user else None
        group_id = str(uninfo.group.id) if uninfo.group else None
        
        # 解析子命令
        if "查看" in message_text:
            info = continuous_manager.get_continuous_info(user_id, group_id)
            msg = f"🔄 当前连战模式设置\n"
            msg += f"状态：{info['status']}\n"
            msg += f"描述：{info['description']}\n"
            msg += f"设置来源：{info['source']}"
            
            # 添加当前连战统计信息
            current_game = get_game_instance().get_game(uninfo)
            if current_game and current_game.get("continuous_mode", False):
                continuous_count = get_game_instance().get_continuous_count(uninfo)
                if continuous_count > 0:
                    msg += f"\n\n📊 当前连战统计\n"
                    msg += f"连战轮数：{continuous_count}轮\n"
                    msg += f"剩余尝试：{get_game_instance().max_attempts - len(current_game['guesses'])}次"
            
            await matcher.send(msg)
            return
        
        elif "重置" in message_text:
            reset_result = continuous_manager.reset_continuous_mode(user_id, group_id)
            if reset_result["success"]:
                msg = f"✅ 连战模式已重置\n"
                msg += f"当前状态：{reset_result['status']}\n"
                msg += f"作用范围：{reset_result['scope']}"
                await matcher.send(msg)
            else:
                await matcher.send(f"❌ 重置失败：{reset_result['message']}")
            return
        
        # 处理设置连战模式
        elif "开启" in message_text:
            set_result = continuous_manager.set_continuous_mode(True, user_id, group_id)
            if set_result["success"]:
                msg = f"✅ 连战模式已开启\n"
                msg += f"作用范围：{set_result['scope']}\n"
                msg += f"💡 猜对后会自动开始下一轮，无需重新输入开始指令"
                await matcher.send(msg)
            else:
                await matcher.send(f"❌ 开启失败：{set_result['message']}")
            return
        
        elif "关闭" in message_text:
            set_result = continuous_manager.set_continuous_mode(False, user_id, group_id)
            if set_result["success"]:
                msg = f"⏹️ 连战模式已关闭\n"
                msg += f"作用范围：{set_result['scope']}\n"
                msg += f"💡 猜对后游戏结束，需要重新输入开始指令"
                await matcher.send(msg)
            else:
                await matcher.send(f"❌ 关闭失败：{set_result['message']}")
            return
        
        # 如果没有提供参数，显示当前设置和帮助
        else:
            info = continuous_manager.get_continuous_info(user_id, group_id)
            msg = f"🔄 当前连战模式设置\n"
            msg += f"状态：{info['status']}\n"
            msg += f"描述：{info['description']}\n"
            msg += f"设置来源：{info['source']}\n\n"
            msg += f"💡 连战模式说明：\n"
            msg += f"🔄 开启：猜对后自动开始下一轮，无需重新输入开始指令\n"
            msg += f"⏹️ 关闭：猜对后游戏结束，需要重新输入开始指令\n\n"
            msg += f"🔧 使用方法：\n"
            msg += f"[arkstart 连战 开启] - 开启连战模式\n"
            msg += f"[arkstart 连战 关闭] - 关闭连战模式\n"
            msg += f"[arkstart 连战 查看] - 查看当前设置\n"
            msg += f"[arkstart 连战 重置] - 重置为默认设置\n\n"
            msg += f"💡 提示：连战模式设置会影响游戏体验"
            await matcher.send(msg)
    
    except FinishedException:
        return
    except Exception as e:
        import traceback
        error_msg = f"处理连战模式设置时出错: {str(e)}\n{traceback.format_exc()}"
        await matcher.send(f"❌ 连战模式设置出错，请检查日志: {str(e)}")

async def handle_pool_settings(uninfo: Uninfo, matcher: Matcher, message_text: str):
    """统一的题库设置处理"""
    try:
        user_id = str(uninfo.user.id) if uninfo.user else None
        group_id = str(uninfo.group.id) if uninfo.group else None
        
        # 解析子命令
        if "查看" in message_text:
            info = pool_manager.get_pool_info(user_id, group_id)
            msg = f"📚 当前题库设置\n"
            msg += f"星级范围：{info['range_display']}星\n"
            msg += f"可选干员：{info['operator_count']}个\n"
            msg += f"设置来源：{info['source']}"
            await matcher.send(msg)
            return
        
        elif "重置" in message_text:
            reset_result = pool_manager.reset_pool_range(user_id, group_id)
            if reset_result["success"]:
                msg = f"✅ 题库已重置\n"
                msg += f"星级范围：{reset_result['range_str']}星\n"
                msg += f"可选干员：{reset_result['operator_count']}个\n"
                msg += f"作用范围：{reset_result['scope']}"
                await matcher.send(msg)
            else:
                await matcher.send("❌ 重置失败")
            return
        
        # 处理设置星级范围
        else:
            # 从消息中提取参数
            import re
            range_match = re.search(r'题库\s+([0-9]+(?:-[0-9]+)?)', message_text)
            range_str = range_match.group(1) if range_match else None
            
            # 如果没有提供参数，显示当前设置和帮助
            if not range_str:
                info = pool_manager.get_pool_info(user_id, group_id)
                msg = f"📚 当前题库设置\n"
                msg += f"星级范围：{info['range_display']}星\n"
                msg += f"可选干员：{info['operator_count']}个\n"
                msg += f"设置来源：{info['source']}\n\n"
                msg += f"💡 题库说明：\n"
                msg += f"• 6星：仅包含6星干员，难度较高\n"
                msg += f"• 4-6星：包含4-6星干员，难度适中\n"
                msg += f"• 1-6星：包含所有星级，难度较低\n\n"
                msg += f"🔧 使用方法：\n"
                msg += f"[arkstart 题库 6] - 设置为6星\n"
                msg += f"[arkstart 题库 4-6] - 设置为4-6星\n"
                msg += f"[arkstart 题库 查看] - 查看当前设置\n"
                msg += f"[arkstart 题库 重置] - 重置为默认设置"
                await matcher.send(msg)
                return
            
            # 设置新的星级范围
            set_result = pool_manager.set_pool_range(user_id, group_id, range_str)
            if set_result["success"]:
                rarity_display = f"{min(set_result['rarity_list'])}-{max(set_result['rarity_list'])}" if len(set_result['rarity_list']) > 1 else str(set_result['rarity_list'][0])
                msg = f"✅ 题库设置成功\n"
                msg += f"星级范围：{rarity_display}星\n"
                msg += f"可选干员：{set_result['operator_count']}个\n"
                msg += f"作用范围：{set_result['scope']}"
                
                # 如果是群聊设置，添加说明
                if group_id:
                    msg += f"\n💡 群聊题库已更新，对本群所有成员生效"
                
                await matcher.send(msg)
            else:
                msg = f"❌ 设置失败\n"
                msg += f"错误：{set_result['error']}\n\n"
                msg += f"💡 正确格式：\n"
                msg += f"6 - 仅6星干员\n"
                msg += f"5-6 - 5至6星干员\n"
                msg += f"1-6 - 全部星级"
                await matcher.send(msg)
    
    except FinishedException:
        return
    except Exception as e:
        import traceback
        error_msg = f"处理题库设置时出错: {str(e)}\n{traceback.format_exc()}"
        await matcher.send(f"❌ 题库设置出错，请检查日志: {str(e)}")

async def handle_mode_settings(uninfo: Uninfo, matcher: Matcher, message_text: str):
    """统一的模式设置处理"""
    try:
        user_id = str(uninfo.user.id) if uninfo.user else None
        group_id = str(uninfo.group.id) if uninfo.group else None
        
        # 解析子命令
        if "查看" in message_text:
            info = mode_manager.get_mode_info(user_id, group_id)
            msg = f"🎭 当前模式设置\n"
            msg += f"模式：{info['mode']}\n"
            msg += f"描述：{info['description']}\n"
            msg += f"设置来源：{info['source']}"
            await matcher.send(msg)
            return
        
        elif "重置" in message_text:
            reset_result = mode_manager.reset_mode(user_id, group_id)
            if reset_result["success"]:
                msg = f"✅ 模式已重置\n"
                msg += f"当前模式：{reset_result['mode']}\n"
                msg += f"作用范围：{reset_result['scope']}"
                await matcher.send(msg)
            else:
                await matcher.send(f"❌ 重置失败：{reset_result['message']}")
            return
        
        # 处理设置模式
        else:
            # 从消息中提取参数
            import re
            mode_match = re.search(r'模式\s+(兔头|大头)', message_text)
            mode = mode_match.group(1) if mode_match else None
            
            # 如果没有提供参数，显示当前设置和帮助
            if not mode:
                info = mode_manager.get_mode_info(user_id, group_id)
                msg = f"🎭 当前模式设置\n"
                msg += f"模式：{info['mode']}\n"
                msg += f"描述：{info['description']}\n"
                msg += f"设置来源：{info['source']}\n\n"
                msg += f"💡 模式说明：\n"
                msg += f"🐰 兔头模式：增加游戏趣味性\n"
                msg += f"👤 大头模式：适合正常游戏体验\n\n"
                msg += f"🔧 使用方法：\n"
                msg += f"[arkstart 模式 大头] - 设置为大头模式\n"
                msg += f"[arkstart 模式 兔头] - 设置为兔头模式\n"
                msg += f"[arkstart 模式 查看] - 查看当前设置\n"
                msg += f"[arkstart 模式 重置] - 重置为默认模式\n\n"
                msg += f"💡 提示：模式设置会影响游戏体验"
                await matcher.send(msg)
                return
            
            # 设置新的模式
            set_result = mode_manager.set_mode(mode, user_id, group_id)
            if set_result["success"]:
                msg = f"✅ 模式设置成功\n"
                msg += f"模式：{set_result['mode']}\n"
                msg += f"作用范围：{set_result['scope']}\n"
                msg += f"描述：{mode_manager._get_mode_description(set_result['mode'])}\n\n"
                
                # 添加模式切换说明
                if set_result['mode'] == "兔头":
                    msg += f"🐰 已切换到兔头模式\n"
                    msg += f"💡 下次开始游戏时将使用兔头模式\n"
                    msg += f"🎨 兔头模式增加了游戏的趣味性"
                else:
                    msg += f"👤 已切换到大头模式\n"
                    msg += f"💡 下次开始游戏时将使用大头模式\n"
                    msg += f"🎯 大头模式适合正常的游戏体验"
                
                if set_result['mode'] == "兔头":
                    # 兔头模式：发送消息和图片
                    try:
                        from pathlib import Path
                        image_path = Path(__file__).parent.parent / "resources" / "images" / "xlpj" / "血狼破军_B站头像.webp"
                        if image_path.exists():
                            with open(image_path, 'rb') as f:
                                await UniMessage([msg, Image(raw=f.read())]).send()
                        else:
                            await matcher.send(msg)
                    except Exception:
                        await matcher.send(msg)
                else:
                    await matcher.send(msg)
            else:
                await matcher.send(f"❌ 设置失败：{set_result['message']}")
    
    except FinishedException:
        return
    except Exception as e:
        import traceback
        error_msg = f"处理模式设置时出错: {str(e)}\n{traceback.format_exc()}"
        await matcher.send(f"❌ 模式设置出错，请检查日志: {str(e)}")

async def handle_end(uninfo: Uninfo):
    game_data = get_game_instance().get_game(uninfo)
    operator = game_data["operator"]
    current_mode = game_data.get("current_mode", "大头")
    get_game_instance().end_game(uninfo)
    img = await render_correct_answer(operator, current_mode)
    await UniMessage(Image(raw=img)).send()

@guess_matcher.handle()
async def handle_guess(uninfo: Uninfo, event: Event):
    guess_name = event.get_plaintext().strip()
    if guess_name in ("", "结束", "arkstart"):
        if guess_name == "结束":
            # 检查是否在连战模式中
            game_data = get_game_instance().get_game(uninfo)
            if game_data and game_data.get("continuous_mode", False):
                continuous_count = get_game_instance().get_continuous_count(uninfo)
                if continuous_count > 0:
                    # 连战模式退出
                    operator = game_data["operator"]
                    current_mode = game_data.get("current_mode", "大头")
                    get_game_instance().end_game(uninfo)
                    img = await render_correct_answer(operator, current_mode)
                    await UniMessage([
                        f"🔄 连战模式已退出\n🎯 正确答案：",
                        Image(raw=img),
                        f"\n📊 本次连战共完成{continuous_count}轮"
                    ]).send()
                else:
                    # 普通游戏结束
                    await handle_end(uninfo)
            else:
                # 普通游戏结束
                await handle_end(uninfo)
        return
    # 检查游戏状态
    game_data = get_game_instance().get_game(uninfo)
    if not game_data:
        return
    # 检查重复猜测
    if any(g["name"] == guess_name for g in game_data["guesses"]):
        await UniMessage.text(f"🤔 已经猜过【{guess_name}】了，请尝试其他干员！").send()
        return
        
    correct, guessed, comparison = get_game_instance().guess(uninfo, guess_name)
    
    if correct:
        # 检查连战模式
        continuous_mode = game_data.get("continuous_mode", False)
        
        if continuous_mode:
            # 连战模式：自动开始新游戏
            # 更新连战计数
            continuous_count = get_game_instance().update_continuous_count(uninfo)
            
            # 结束当前游戏
            get_game_instance().end_game(uninfo)
            
            # 开始新游戏
            new_game = get_game_instance().start_new_game(uninfo)
            
            # 显示答案并提示连战模式
            current_mode = game_data.get("current_mode", "大头")
            img = await render_correct_answer(guessed, current_mode)
            
            # 构建连战模式提示消息
            continuous_msg = f"🎉 恭喜你猜对了！\n🎯 正确答案："
            
            if continuous_count > 1:
                continuous_msg += f"\n🔄 连战进度：第{continuous_count}轮"
            else:
                continuous_msg += f"\n🔄 连战模式已启动"
            
            continuous_msg += f"\n💡 直接输入干员名即可开始下一轮猜测"
            continuous_msg += f"\n⏹️ 输入「结束」可退出连战模式"
            
            await UniMessage([
                continuous_msg,
                Image(raw=img)
            ]).send()
        else:
            # 普通模式：正常结束
            get_game_instance().end_game(uninfo)
            current_mode = game_data.get("current_mode", "大头")
            img = await render_correct_answer(guessed, current_mode)
            await UniMessage([
                "🎉 恭喜你猜对了！\n🎯 正确答案：",
                Image(raw=img)
            ]).send()
        return
    
    if not guessed:
        similar = get_game_instance().find_similar_operators(guess_name)
        if not similar:
            return
        err_msg = f"❓ 未找到干员【{guess_name}】！\n💡 尝试以下相似结果：" + "、".join(similar)
        await UniMessage.text(err_msg).send()
        return

    attempts_left = get_game_instance().max_attempts - len(game_data["guesses"])
    # 检查尝试次数
    if attempts_left <= 0:
        operator = game_data["operator"]
        current_mode = game_data.get("current_mode", "大头")
        get_game_instance().end_game(uninfo)
        img = await render_correct_answer(operator, current_mode)
        await UniMessage([
            "😅 尝试次数已用尽！\n🎯 正确答案：",
            Image(raw=img)
        ]).send()
        return
    
    current_mode = game_data.get("current_mode", "大头")
    img = await render_guess_result(guessed, comparison, attempts_left, current_mode)
    
    # 添加连战模式进度显示
    if get_game_instance().is_continuous_mode(uninfo):
        continuous_count = get_game_instance().get_continuous_count(uninfo)
        if continuous_count > 0:
            # 在图片下方添加连战进度提示
            progress_msg = f"\n🔄 连战进度：第{continuous_count}轮 | 剩余尝试：{attempts_left}次"
            await UniMessage([
                Image(raw=img),
                progress_msg
            ]).send()
        else:
            await UniMessage(Image(raw=img)).send()
    else:
        await UniMessage(Image(raw=img)).send()

async def handle_update_resources(uninfo: Uninfo, matcher: Matcher, message_text: str):
    """统一的资源更新处理"""
    try:
        import re
        
        # 解析更新类型
        update_match = re.search(r'更新\s+(.+?)(?:\s|$)', message_text)
        update_type = update_match.group(1).strip() if update_match else None
        
        if not update_type:
            # 显示更新帮助信息
            msg = f"🔄 资源更新系统\n\n"
            msg += f"💡 更新类型说明：\n"
            msg += f"• 数据库 - 更新干员数据库（从PRTS Wiki获取最新数据）\n"
            msg += f"• 立绘 - 更新所有干员立绘\n"
            msg += f"• 1星 - 只更新1星干员立绘\n"
            msg += f"• 2星 - 只更新2星干员立绘\n"
            msg += f"• 3星 - 只更新3星干员立绘\n"
            msg += f"• 4星 - 只更新4星干员立绘\n"
            msg += f"• 5星 - 只更新5星干员立绘\n"
            msg += f"• 6星 - 只更新6星干员立绘\n"
            msg += f"• 1-3星 - 更新1-3星干员立绘\n"
            msg += f"• 4-6星 - 更新4-6星干员立绘\n"
            msg += f"• 全量 - 更新数据库和所有立绘\n\n"
            msg += f"🔧 使用方法：\n"
            msg += f"[arkstart 更新 数据库] - 只更新干员数据库\n"
            msg += f"[arkstart 更新 6星] - 只更新6星干员立绘\n"
            msg += f"[arkstart 更新 4-6星] - 更新4-6星干员立绘\n"
            msg += f"[arkstart 更新 全量] - 全量更新（数据库+立绘）\n\n"
            msg += f"⚠️ 注意：立绘更新可能需要较长时间，请耐心等待"
            await matcher.send(msg)
            return
        
        # 发送开始更新提示
        await matcher.send(f"🔄 开始更新资源...\n📋 更新类型：{update_type}\n⏳ 请稍候，这可能需要几分钟时间...")
        
        try:
            if update_type == "数据库":
                # 更新干员数据库
                await update_database(matcher)
            elif update_type == "全量":
                # 全量更新：先更新数据库，再更新立绘
                await update_database(matcher)
                # 数据库更新完成后，重新加载数据
                reload_success = get_game_instance().reload_data()
                if reload_success:
                    await matcher.send("🔄 全量更新：数据库更新完成，数据已重新加载\n🎨 继续更新立绘资源...")
                else:
                    await matcher.send("⚠️ 全量更新：数据库更新完成，但数据重新加载失败\n🎨 继续更新立绘资源...")
                
                await update_illustrations(matcher, "1-6")
                
                # 全量更新完成后，再次重载数据以确保所有资源都是最新的
                final_reload_success = get_game_instance().reload_data()
                if final_reload_success:
                    await matcher.send("🎉 全量更新完成！\n🔄 数据库和立绘资源已更新，数据已重新加载\n✅ 现在可以正常使用所有功能了")
                else:
                    await matcher.send("⚠️ 全量更新完成，但最终数据重新加载失败\n💡 请尝试重新启动插件或联系管理员")
            elif update_type in ["立绘", "1星", "2星", "3星", "4星", "5星", "6星"]:
                # 更新指定星级的立绘
                rarity_map = {
                    "立绘": "1-6",
                    "1星": "1",
                    "2星": "2", 
                    "3星": "3",
                    "4星": "4",
                    "5星": "5",
                    "6星": "6"
                }
                await update_illustrations(matcher, rarity_map[update_type])
            elif re.match(r'^\d+-\d+星$', update_type):
                # 更新指定星级范围的立绘
                await update_illustrations(matcher, update_type.replace("星", ""))
            else:
                await matcher.send(f"❌ 不支持的更新类型：{update_type}\n💡 请使用 /arkstart 更新 查看支持的更新类型")
                return
                
        except Exception as e:
            await matcher.send(f"❌ 更新过程中出错：{str(e)}\n💡 请检查日志或稍后重试")
            return
            
    except Exception as e:
        await matcher.send(f"❌ 资源更新出错：{str(e)}")

async def update_database(matcher: Matcher):
    """更新干员数据库"""
    try:
        # 导入更新模块
        import sys
        from pathlib import Path
        
        # 添加resource_tools到Python路径
        resource_tools_path = Path(__file__).parent.parent / "resource_tools"
        sys.path.insert(0, str(resource_tools_path))
        
        from update_simple import update_data
        
        await matcher.send("📊 正在更新干员数据库...\n⏳ 请稍候，这可能需要几分钟时间...")
        
        # 运行数据库更新
        success = await update_data()
        
        if success:
            # 数据库更新成功后，重新加载数据
            reload_success = get_game_instance().reload_data()
            
            if reload_success:
                await matcher.send("✅ 干员数据库更新完成！\n📚 已获取最新的干员信息\n🔄 数据已重新加载，现在可以开始游戏了")
            else:
                await matcher.send("⚠️ 干员数据库更新完成，但数据重新加载失败\n💡 请尝试重新启动插件或联系管理员")
        else:
            await matcher.send("❌ 干员数据库更新失败，请检查日志")
        
    except Exception as e:
        await matcher.send(f"❌ 数据库更新失败：{str(e)}")
        raise

async def update_illustrations(matcher: Matcher, rarity_range: str):
    """更新干员立绘"""
    try:
        # 导入立绘下载模块
        import sys
        from pathlib import Path
        
        # 添加resource_tools到Python路径
        resource_tools_path = Path(__file__).parent.parent / "resource_tools"
        sys.path.insert(0, str(resource_tools_path))
        
        # 直接运行立绘下载脚本
        import subprocess
        import asyncio
        
        # 根据星级范围显示信息
        if "-" in rarity_range:
            min_rarity, max_rarity = map(int, rarity_range.split("-"))
            await matcher.send(f"🎨 正在更新{min_rarity}-{max_rarity}星干员立绘...\n⏳ 请耐心等待...")
        else:
            min_rarity = max_rarity = int(rarity_range)
            await matcher.send(f"🎨 正在更新{min_rarity}星干员立绘...\n⏳ 请耐心等待...")
        
        # 运行立绘下载脚本
        script_path = Path(__file__).parent.parent / "resource_tools" / "illustration_downloader_v2.py"
        
        # 使用asyncio创建子进程
        process = await asyncio.create_subprocess_exec(
            sys.executable, str(script_path),
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE
        )
        
        stdout, stderr = await process.communicate()
        
        if process.returncode == 0:
            # 立绘更新完成后，重新加载数据以确保立绘路径信息是最新的
            reload_success = get_game_instance().reload_data()
            
            # 发送完成消息
            if min_rarity == max_rarity:
                msg = f"✅ {min_rarity}星干员立绘更新完成！\n"
            else:
                msg = f"✅ {min_rarity}-{max_rarity}星干员立绘更新完成！\n"
            
            if reload_success:
                msg += f"🔄 数据已重新加载，立绘资源现在可用\n"
            else:
                msg += f"⚠️ 立绘更新完成，但数据重新加载失败\n"
            
            msg += f"📊 更新完成，请检查日志获取详细信息"
            
            await matcher.send(msg)
        else:
            error_msg = stderr.decode('utf-8', errors='ignore')
            await matcher.send(f"❌ 立绘更新失败：\n{error_msg}")
        
    except Exception as e:
        await matcher.send(f"❌ 立绘更新失败：{str(e)}")
        raise 


"""
明日方舟猜干员游戏插件
支持多种游戏模式和题库设置
"""

from .game_tools import *

# 插件元数据
__version__ = "0.3.1"
__author__ = "lizhiqi233-rgb"
__description__ = "明日方舟猜干员游戏 - 支持多种游戏模式和题库设置"

# 导出插件元数据
__all__ = ["__plugin_meta__"]



"""
资源工具模块
包含所有资源获取相关的功能模块
"""

from . import illustration_config
from . import illustration_downloader_v2
from . import run_illustration_download
from . import update_simple

__all__ = [
    'illustration_config',
    'illustration_downloader_v2',
    'run_illustration_download',
    'update_simple'
]



"""
题库管理器
负责管理干员抽取的星级范围设置
"""

import re
import json
from typing import List, Tuple, Dict, Any
import csv
from pathlib import Path
from nonebot import require

# 加载 localstore 插件
require("nonebot_plugin_localstore")
import nonebot_plugin_localstore as store

class PoolManager:
    """题库管理器"""
    
    def __init__(self):
        self.data_file = store.get_plugin_data_file("pool_settings.json")
        # 动态星级数量统计缓存，由 characters.csv 加载
        self._rarity_counts: Dict[int, int] = {}
        self._load_rarity_counts()
        self._load_settings()
    
    def _load_settings(self):
        """加载设置文件"""
        try:
            if self.data_file.exists():
                self.settings = json.loads(self.data_file.read_text(encoding='utf-8'))
            else:
                self.settings = {}
        except Exception as e:
            print(f"加载题库设置失败: {e}")
            self.settings = {}
    
    def _save_settings(self):
        """保存设置文件"""
        try:
            self.data_file.write_text(json.dumps(self.settings, ensure_ascii=False, indent=2), encoding='utf-8')
        except Exception as e:
            print(f"保存题库设置失败: {e}")

    def _load_rarity_counts(self) -> None:
        """
        从 update_simple.py 生成的 characters.csv 动态统计各星级干员数量。
        若文件不存在或解析失败，不抛异常，保留为空以便使用兜底估算。
        """
        try:
            characters_csv: Path = store.get_plugin_data_file("characters.csv")
            if not characters_csv.exists():
                self._rarity_counts = {}
                return
            counts: Dict[int, int] = {}
            with characters_csv.open("r", encoding="utf-8-sig", newline="") as f:
                reader = csv.DictReader(f)
                for row in reader:
                    rarity_str = row.get("rarity")
                    if rarity_str is None:
                        continue
                    try:
                        rarity = int(rarity_str)
                    except ValueError:
                        continue
                    counts[rarity] = counts.get(rarity, 0) + 1
            self._rarity_counts = counts
        except Exception:
            # 任何异常都不影响主流程，使用兜底估算
            self._rarity_counts = {}
    
    def refresh_rarity_counts(self) -> None:
        """对外暴露的刷新接口：重新从 characters.csv 统计星级数量"""
        self._load_rarity_counts()
    
    def parse_rarity_range(self, range_str: str) -> List[int]:
        """
        解析星级范围字符串
        
        Args:
            range_str: 星级范围字符串，如 "6", "5-6", "4-6", "1-6"
            
        Returns:
            星级列表，如 [6] 或 [5, 6] 或 [4, 5, 6]
            
        Raises:
            ValueError: 范围格式错误或超出有效范围
        """
        range_str = range_str.strip()
        
        # 单个数字
        if range_str.isdigit():
            rarity = int(range_str)
            if 1 <= rarity <= 6:
                return [rarity]
            else:
                raise ValueError(f"星级必须在1-6之间，得到: {rarity}")
        
        # 范围格式 x-y
        range_match = re.match(r'^(\d+)-(\d+)$', range_str)
        if range_match:
            start, end = int(range_match.group(1)), int(range_match.group(2))
            
            if not (1 <= start <= 6 and 1 <= end <= 6):
                raise ValueError(f"星级必须在1-6之间，得到: {start}-{end}")
            
            if start > end:
                raise ValueError(f"起始星级不能大于结束星级: {start}-{end}")
            
            return list(range(start, end + 1))
        
        raise ValueError(f"无效的星级范围格式: {range_str}，支持格式: '6' 或 '4-6'")
    
    def set_pool_range(self, user_id: str, group_id: str, range_str: str) -> Dict[str, Any]:
        """
        设置用户/群组的题库范围
        
        设计理念:
        - 群聊: 所有成员共享一个题库设置，任何成员都可以修改
        - 私聊: 每个用户有独立的个人设置
        - 优先级: 群聊设置 > 个人设置 > 默认设置(6星)
        
        Args:
            user_id: 用户ID
            group_id: 群组ID (私聊时为None)
            range_str: 星级范围字符串
            
        Returns:
            设置结果字典，包含成功状态、星级列表、干员数量等信息
        """
        try:
            rarity_list = self.parse_rarity_range(range_str)
            
            # 构建设置键 (群聊优先)
            key = f"group_{group_id}" if group_id else f"user_{user_id}"
            
            # 保存设置
            self.settings[key] = {
                "rarity_range": rarity_list,
                "range_str": range_str
            }
            self._save_settings()
            
            # 统计对应星级的干员数量
            operator_count = self._count_operators_by_rarity(rarity_list)
            
            return {
                "success": True,
                "rarity_list": rarity_list,
                "range_str": range_str,
                "operator_count": operator_count,
                "scope": "群聊" if group_id else "个人"
            }
            
        except ValueError as e:
            return {
                "success": False,
                "error": str(e)
            }
    
    def get_pool_range(self, user_id: str, group_id: str) -> List[int]:
        """
        获取用户/群组的题库范围
        
        Args:
            user_id: 用户ID
            group_id: 群组ID (私聊时为None)
            
        Returns:
            星级列表，默认为 [6]
        """
        # 群聊设置优先于个人设置
        key = f"group_{group_id}" if group_id else f"user_{user_id}"
        
        if key in self.settings:
            return self.settings[key]["rarity_range"]
        
        # 返回默认设置 (只选6星)
        return [6]
    
    def get_pool_info(self, user_id: str, group_id: str) -> Dict[str, Any]:
        """
        获取当前题库设置信息
        
        Args:
            user_id: 用户ID  
            group_id: 群组ID (私聊时为None)
            
        Returns:
            题库信息字典
        """
        rarity_list = self.get_pool_range(user_id, group_id)
        
        # 构建范围字符串显示
        if len(rarity_list) == 1:
            range_display = str(rarity_list[0])
        else:
            range_display = f"{min(rarity_list)}-{max(rarity_list)}"
        
        # 统计干员数量
        operator_count = self._count_operators_by_rarity(rarity_list)
        
        # 判断设置来源
        group_key = f"group_{group_id}" if group_id else None
        user_key = f"user_{user_id}"
        
        if group_key and group_key in self.settings:
            source = "群聊设置"
        elif user_key in self.settings:
            source = "个人设置"
        else:
            source = "默认设置"
        
        return {
            "rarity_list": rarity_list,
            "range_display": range_display,
            "operator_count": operator_count,
            "source": source
        }
    
    def reset_pool_range(self, user_id: str, group_id: str) -> Dict[str, Any]:
        """
        重置题库范围为默认设置
        
        Args:
            user_id: 用户ID
            group_id: 群组ID (私聊时为None)
            
        Returns:
            重置结果字典
        """
        key = f"group_{group_id}" if group_id else f"user_{user_id}"
        
        if key in self.settings:
            del self.settings[key]
            self._save_settings()
            
        return {
            "success": True,
            "rarity_list": [6],
            "range_str": "6",
            "operator_count": self._count_operators_by_rarity([6]),
            "scope": "群聊" if group_id else "个人"
        }
    
    def _count_operators_by_rarity(self, rarity_list: List[int]) -> int:
        """
        统计指定星级范围内的干员数量
        
        Args:
            rarity_list: 星级列表
            
        Returns:
            干员数量
        """
        # 仅使用动态统计结果；无数据返回0
        if not self._rarity_counts:
            return 0
        return sum(self._rarity_counts.get(r, 0) for r in rarity_list)

# 全局题库管理器实例
pool_manager = PoolManager()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
明日方舟立绘下载配置文件
"""

import os
from pathlib import Path

# 在独立脚本环境中，导入 nonebot_plugin_localstore 可能因 NoneBot 未初始化而失败
def _safe_get_data_dir() -> Path:
    try:
        from nonebot_plugin_localstore import get_plugin_data_dir as _get_dir  # type: ignore
        return _get_dir()
    except Exception:
        # 回退到 localstore 的默认路径结构
        return Path.home() / ".local" / "share" / "nonebot2" / "nonebot_plugin_arkguesser"

# 基础配置：使用 localstore 提供的插件数据目录
DATA_DIR = _safe_get_data_dir()
OUTPUT_DIR = DATA_DIR / "illustrations"

# 干员数据文件路径（与更新脚本保持一致，位于 DATA_DIR 根目录）
CHARACTERS_CSV_PATH = DATA_DIR / "characters.csv"

# 下载配置
DOWNLOAD_CONFIG = {
    "max_concurrent": 3,  # 最大并发下载数
    "timeout": 60,         # 请求超时时间（秒）
    "retry_times": 3,      # 重试次数
    "delay_between_requests": 0.5,  # 请求间隔（秒）
}

# PRTS Wiki API 配置
PRTS_CONFIG = {
    "base_url": "https://prts.wiki/api.php",
    "assets_url": "https://torappu.prts.wiki/assets",
    "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
}

# 立绘类型配置
ILLUSTRATION_TYPES = {
    "半身像": {
        "folder": "char_portrait",
        "suffixes": ["_1", "_2"],  # 精英化等级后缀
        "extensions": [".png", ".jpg", ".webp"]
    },
    "头像": {
        "folder": "char_avatar",
        "suffixes": [""],  # 无后缀
        "extensions": [".png", ".jpg", ".webp"]
    },
    "皮肤立绘": {
        "folder": "char_portrait",
        "suffixes": ["_skin1", "_skin2"],  # 皮肤后缀
        "extensions": [".png", ".jpg", ".webp"]
    }
}

# 日志配置
LOG_CONFIG = {
    "level": "INFO",
    "format": "{time:YYYY-MM-DD HH:mm:ss} | {level} | {message}",
    "file": "illustration_download.log",
    "rotation": "1 day",
    "retention": "7 days"
}

# 文件命名配置
NAMING_CONFIG = {
    "separator": "_",
    "include_rarity": False,  # 不包含稀有度信息
    "include_career": False,
    "include_level": True,
    "filename_format": "{name}_{type}_{level}"
}

# 过滤配置
FILTER_CONFIG = {
    "min_rarity": 1,      # 最小稀有度
    "max_rarity": 6,      # 最大稀有度
    "include_careers": [   # 包含的职业
        "先锋", "近卫", "重装", "狙击", "术师", "医疗", "辅助", "特种"
    ],
    "exclude_operators": [  # 排除的干员ID
        # 可以在这里添加不需要下载的干员ID
    ]
}

# 输出配置
OUTPUT_CONFIG = {
    "create_subdirectories": True,  # 是否创建子目录
    "subdirectory_structure": {     # 子目录结构
        "by_rarity": True,          # 按稀有度分类
        "by_career": False,         # 按职业分类
        "by_type": True             # 按立绘类型分类
    },
    "save_metadata": True,          # 是否保存元数据
    "metadata_format": "json"       # 元数据格式
}

# 验证配置
VALIDATION_CONFIG = {
    "verify_image_integrity": True,  # 验证图片完整性
    "min_file_size": 1024,           # 最小文件大小（字节）
    "max_file_size": 10 * 1024 * 1024,  # 最大文件大小（字节）
    "allowed_formats": ["png", "jpg", "jpeg", "webp"]  # 允许的图片格式
}

# 网络配置
NETWORK_CONFIG = {
    "use_proxy": False,              # 是否使用代理
    "proxy": {                       # 代理配置
        "http": None,
        "https": None
    },
    "verify_ssl": False,             # 是否验证SSL证书
    "follow_redirects": True,        # 是否跟随重定向
    "max_redirects": 5               # 最大重定向次数
}

# 错误处理配置
ERROR_HANDLING_CONFIG = {
    "continue_on_error": True,       # 遇到错误时是否继续
    "log_errors": True,              # 是否记录错误
    "save_failed_list": True,        # 是否保存失败列表
    "retry_failed": True,            # 是否重试失败的下载
    "max_retry_attempts": 3          # 最大重试次数
}

# 性能配置
PERFORMANCE_CONFIG = {
    "chunk_size": 8192,              # 下载块大小
    "buffer_size": 1024 * 1024,     # 缓冲区大小
    "use_async_io": True,            # 是否使用异步IO
    "memory_limit": 100 * 1024 * 1024  # 内存限制（字节）
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
明日方舟猜谜游戏数据自动更新程序 - 简化版本
基于 arknights-toolkit 的 main.py 原理
"""

import re
import json
import csv
import asyncio
import sys
from pathlib import Path
def _safe_get_data_dir() -> Path:
    try:
        from nonebot_plugin_localstore import get_plugin_data_dir as _get_dir  # type: ignore
        return _get_dir()
    except Exception:
        return Path.home() / ".local" / "share" / "nonebot2" / "nonebot_plugin_arkguesser"
from typing import Dict, List, Optional

import httpx
from nonebot import logger

# 强制设置系统编码为UTF-8
if sys.platform.startswith('win'):
    import locale
    locale.setlocale(locale.LC_ALL, 'zh_CN.UTF-8')

# 插件不应修改用户日志配置，移除自定义日志处理器

# 数据文件路径 - 使用 localstore 插件数据目录（根目录）
DATA_DIR = _safe_get_data_dir()
CAMP_FILE = DATA_DIR / "camp.json"
CAREER_FILE = DATA_DIR / "career.json"
CHARACTERS_FILE = DATA_DIR / "characters.csv"

# 维基百科API
WIKI_API = "https://prts.wiki/api.php"

# 正则表达式（基于arknights-toolkit）
ID_PATTERN = re.compile(r"\|干员id=char_([^|]+?)\n\|")
RARITY_PATTERN = re.compile(r"\|稀有度=(\d+?)\n\|")
CHAR_PATTERN = re.compile(r"\|职业=([^|]+?)\n\|")
SUB_CHAR_PATTERN = re.compile(r"\|分支=([^|]+?)\n\|")
RACE_PATTERN = re.compile(r"\|种族=([^|]+?)\n\|")
ORG_PATTERN = re.compile(r"\|所属国家=([^|]+?)\n\|")
ORG_PATTERN1 = re.compile(r"\|所属组织=([^|]+?)\n\|")
ORG_PATTERN2 = re.compile(r"\|所属团队=([^|]+?)\n\|")
ART_PATTERN = re.compile(r"\|画师=([^|]+?)\n\|")
NAME_PATTERN = re.compile(r"\|干员名=([^|]+?)\n\|")
POSITION_PATTERN = re.compile(r"\|位置=([^|]+?)\n\|")
TAG_PATTERN = re.compile(r"\|标签=([^|]+?)\n\|")
JAPANESE_VOICE_PATTERN = re.compile(r"\|日文配音=([^|]+?)\n\|")
OBTAIN_METHOD_PATTERN = re.compile(r"\|获得方式=([^|]+?)\n\|")
ONLINE_TIME_PATTERN = re.compile(r"\|上线时间=([^|}]+?)(?:\n\||\}|\n)")
# 修改部署费用正则表达式，提取最后一个阶段的数字
DEPLOY_COST_PATTERN = re.compile(r"\|部署费用=(?:\d+→)*(\d+)\n\|")
# 添加更多可能的部署费用字段名，都提取最后一个阶段的数字
DEPLOY_COST_PATTERN2 = re.compile(r"\|费用=(?:\d+→)*(\d+)\n\|")
DEPLOY_COST_PATTERN3 = re.compile(r"\|cost=(?:\d+→)*(\d+)\n\|")
DEPLOY_COST_PATTERN4 = re.compile(r"\|部署费用_精英0=(\d+)\n\|")
DEPLOY_COST_PATTERN5 = re.compile(r"\|费用_精英0=(\d+)\n\|")
# 阻挡数正则表达式，支持多阶段格式（如：2→3→3）
BLOCK_COUNT_PATTERN = re.compile(r"\|阻挡数=(?:\d+→)*(\d+)\n\|")
ATTACK_SPEED_PATTERN = re.compile(r"\|攻击速度=([^|]+?)\n\|")

# 多阶段属性正则表达式（用于获取干员的满级数据，按精英阶段优先级）
ELITE0_HP_PATTERN = re.compile(r"\|精英0_满级_生命上限=(\d+?)\n\|")
ELITE0_ATK_PATTERN = re.compile(r"\|精英0_满级_攻击=(\d+?)\n\|")
ELITE0_DEF_PATTERN = re.compile(r"\|精英0_满级_防御=(\d+?)\n\|")
ELITE0_MAGIC_RES_PATTERN = re.compile(r"\|精英0_满级_法术抗性=(\d+?)\n\|")

ELITE1_HP_PATTERN = re.compile(r"\|精英1_满级_生命上限=(\d+?)\n\|")
ELITE1_ATK_PATTERN = re.compile(r"\|精英1_满级_攻击=(\d+?)\n\|")
ELITE1_DEF_PATTERN = re.compile(r"\|精英1_满级_防御=(\d+?)\n\|")
ELITE1_MAGIC_RES_PATTERN = re.compile(r"\|精英1_满级_法术抗性=(\d+?)\n\|")

ELITE2_HP_PATTERN = re.compile(r"\|精英2_满级_生命上限=(\d+?)\n\|")
ELITE2_ATK_PATTERN = re.compile(r"\|精英2_满级_攻击=(\d+?)\n\|")
ELITE2_DEF_PATTERN = re.compile(r"\|精英2_满级_防御=(\d+?)\n\|")
ELITE2_MAGIC_RES_PATTERN = re.compile(r"\|精英2_满级_法术抗性=(\d+?)\n\|")

GENDER_PATTERN = re.compile(r"\|性别=([^|]+?)\n\|")
BIRTHPLACE_PATTERN = re.compile(r"\|出身地=([^|]+?)\n\|")
BIRTHDAY_PATTERN = re.compile(r"\|生日=([^|]+?)\n\|")
HEIGHT_PATTERN = re.compile(r"\|身高=([^|]+?)\n\|")

# 技能相关正则表达式（支持两种格式：传统{{技能模板和新{{技能2模板）
# 技能1相关
SKILL1_NAME_PATTERN = re.compile(r"'''技能1[（(]精英0开放[）)]'''\s*{{技能[^}]*?\|技能名=([^|]+?)\n\|")
SKILL1_TYPE1_PATTERN = re.compile(r"'''技能1[（(]精英0开放[）)]'''\s*{{技能[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型1=([^|]+?)\n\|")
SKILL1_TYPE2_PATTERN = re.compile(r"'''技能1[（(]精英0开放[）)]'''\s*{{技能[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型2=([^|]+?)\n\|")
SKILL1_SPEC3_INITIAL_PATTERN = re.compile(r"\|技能专精3初始=(\d+?)\n\|")
SKILL1_SPEC3_COST_PATTERN = re.compile(r"\|技能专精3消耗=(\d+?)\n\|")
SKILL1_SPEC3_DURATION_PATTERN = re.compile(r"\|技能专精3持续=([^|}]+?)(?:\n\||\}|\n)")

# 技能2相关
SKILL2_NAME_PATTERN = re.compile(r"'''技能2[（(]精英1开放[）)]'''\s*{{技能[^}]*?\|技能名=([^|]+?)\n\|")
SKILL2_TYPE1_PATTERN = re.compile(r"'''技能2[（(]精英1开放[）)]'''\s*{{技能[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型1=([^|]+?)\n\|")
SKILL2_TYPE2_PATTERN = re.compile(r"'''技能2[（(]精英1开放[）)]'''\s*{{技能[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型2=([^|]+?)\n\|")
SKILL2_SPEC3_INITIAL_PATTERN = re.compile(r"\|技能专精3初始=(\d+?)\n\|")
SKILL2_SPEC3_COST_PATTERN = re.compile(r"\|技能专精3消耗=(\d+?)\n\|")
SKILL2_SPEC3_DURATION_PATTERN = re.compile(r"\|技能专精3持续=([^|}]+?)(?:\n\||\}|\n)")

# 技能3相关
SKILL3_NAME_PATTERN = re.compile(r"'''技能3[（(]精英2开放[）)]'''\s*{{技能[^}]*?\|技能名=([^|]+?)\n\|")
SKILL3_TYPE1_PATTERN = re.compile(r"'''技能3[（(]精英2开放[）)]'''\s*{{技能[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型1=([^|]+?)\n\|")
SKILL3_TYPE2_PATTERN = re.compile(r"'''技能3[（(]精英2开放[）)]'''\s*{{技能[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型2=([^|]+?)\n\|")
SKILL3_SPEC3_INITIAL_PATTERN = re.compile(r"\|技能专精3初始=(\d+?)\n\|")
SKILL3_SPEC3_COST_PATTERN = re.compile(r"\|技能专精3消耗=(\d+?)\n\|")
SKILL3_SPEC3_DURATION_PATTERN = re.compile(r"\|技能专精3持续=([^|}]+?)(?:\n\||\}|\n)")

# 新增：支持{{技能2模板的技能专精3相关正则表达式
# 技能1相关（{{技能2模板）
SKILL1_NAME_PATTERN2 = re.compile(r"'''技能1[（(]精英0开放[）)]'''\s*{{技能2[^}]*?\|技能名=([^|]+?)\n\|")
SKILL1_TYPE1_PATTERN2 = re.compile(r"'''技能1[（(]精英0开放[）)]'''\s*{{技能2[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型1=([^|]+?)\n\|")
SKILL1_TYPE2_PATTERN2 = re.compile(r"'''技能1[（(]精英0开放[）)]'''\s*{{技能2[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型2=([^|]+?)\n\|")
SKILL1_SPEC3_INITIAL_PATTERN2 = re.compile(r"\|技能专精3初始=(\d+?)\n\|")
SKILL1_SPEC3_COST_PATTERN2 = re.compile(r"\|技能专精3消耗=(\d+?)\n\|")
SKILL1_SPEC3_DURATION_PATTERN2 = re.compile(r"\|技能专精3持续=([^|}]+?)(?:\n\||\}|\n)")

# 技能2相关（{{技能2模板）
SKILL2_NAME_PATTERN2 = re.compile(r"'''技能2[（(]精英1开放[）)]'''\s*{{技能2[^}]*?\|技能名=([^|]+?)\n\|")
SKILL2_TYPE1_PATTERN2 = re.compile(r"'''技能2[（(]精英1开放[）)]'''\s*{{技能2[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型1=([^|]+?)\n\|")
SKILL2_TYPE2_PATTERN2 = re.compile(r"'''技能2[（(]精英1开放[）)]'''\s*{{技能2[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型2=([^|]+?)\n\|")
SKILL2_SPEC3_INITIAL_PATTERN2 = re.compile(r"\|技能专精3初始=(\d+?)\n\|")
SKILL2_SPEC3_COST_PATTERN2 = re.compile(r"\|技能专精3消耗=(\d+?)\n\|")
SKILL2_SPEC3_DURATION_PATTERN2 = re.compile(r"\|技能专精3持续=([^|}]+?)(?:\n\||\}|\n)")

# 技能3相关（{{技能2模板）
SKILL3_NAME_PATTERN2 = re.compile(r"'''技能3[（(]精英2开放[）)]'''\s*{{技能2[^}]*?\|技能名=([^|]+?)\n\|")
SKILL3_TYPE1_PATTERN2 = re.compile(r"'''技能3[（(]精英2开放[）)]'''\s*{{技能2[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型1=([^|]+?)\n\|")
SKILL3_TYPE2_PATTERN2 = re.compile(r"'''技能3[（(]精英2开放[）)]'''\s*{{技能2[^}]*?\|技能名=[^|]+?\n\|[^}]*?\|技能类型2=([^|]+?)\n\|")
SKILL3_SPEC3_INITIAL_PATTERN2 = re.compile(r"\|技能专精3初始=(\d+?)\n\|")
SKILL3_SPEC3_COST_PATTERN2 = re.compile(r"\|技能专精3消耗=(\d+?)\n\|")
SKILL3_SPEC3_DURATION_PATTERN2 = re.compile(r"\|技能专精3持续=([^|}]+?)(?:\n\||\}|\n)")

def extract_skill_spec3_initial(content: str, skill_num: int) -> str:
    """
    提取技能专精3初始值
    
    Args:
        content: 页面内容
        skill_num: 技能编号（1、2或3）
    
    Returns:
        技能专精3初始值，如果未找到则返回"未知"
    """
    field_name = f"|技能专精3初始="
    start_pos = content.find(field_name)
    
    if start_pos == -1:
        return "未知"
    
    start_pos += len(field_name)
    
    # 查找换行符
    end_pos = content.find('\n', start_pos)
    if end_pos == -1:
        end_pos = len(content)
    
    value = content[start_pos:end_pos].strip()
    return value if value else "未知"

def extract_skill_spec3_cost(content: str, skill_num: int) -> str:
    """
    提取技能专精3消耗值
    
    Args:
        content: 页面内容
        skill_num: 技能编号（1、2或3）
    
    Returns:
        技能专精3消耗值，如果未找到则返回"未知"
    """
    field_name = f"|技能专精3消耗="
    start_pos = content.find(field_name)
    
    if start_pos == -1:
        return "未知"
    
    start_pos += len(field_name)
    
    # 查找换行符
    end_pos = content.find('\n', start_pos)
    if end_pos == -1:
        end_pos = len(content)
    
    value = content[start_pos:end_pos].strip()
    return value if value else "未知"

def extract_skill_spec3_duration(content: str, skill_num: int) -> str:
    """
    提取技能专精3持续值
    
    Args:
        content: 页面内容
        skill_num: 技能编号（1、2或3）
    
    Returns:
        技能专精3持续值，如果未找到则返回"未知"
    """
    field_name = f"|技能专精3持续="
    start_pos = content.find(field_name)
    
    if start_pos == -1:
        return "未知"
    
    start_pos += len(field_name)
    
    # 查找换行符或管道符
    end_pos = content.find('\n', start_pos)
    pipe_pos = content.find('|', start_pos)
    
    if end_pos == -1 and pipe_pos == -1:
        end_pos = len(content)
    elif end_pos == -1:
        end_pos = pipe_pos
    elif pipe_pos == -1:
        pass  # 使用end_pos
    else:
        end_pos = min(end_pos, pipe_pos)
    
    value = content[start_pos:end_pos].strip()
    return value if value else "未知"

async def get_operator_info(name: str, client: httpx.AsyncClient) -> Optional[Dict]:
    """获取干员信息（基于arknights-toolkit的逻辑）"""
    try:
        # 使用查询API获取页面内容
        query_url = (
            f"{WIKI_API}?action=query&format=json&prop=revisions"
            f"&titles={name}&rvprop=content&rvslots=main"
        )
        
        response = await client.get(query_url)
        response.raise_for_status()
        data = response.json()
        
        # 检查页面是否存在
        if "query" in data and "pages" in data["query"]:
            pages = data["query"]["pages"]
            
            for page_id, page_info in pages.items():
                if page_id != "-1":  # 页面存在
                    if "revisions" in page_info:
                        revisions = page_info["revisions"]
                        if revisions and "slots" in revisions[0]:
                            content = revisions[0]["slots"]["main"]["*"]
                            
                            # 检查获得方式，过滤掉集成战略专属干员
                            obtain_match = re.search(r"\|获得方式=([^|]+?)\n\|", content)
                            if obtain_match:
                                obtain_method = obtain_match.group(1)
                                # 过滤掉获得方式为"无"的干员（集成战略专属）
                                if obtain_method == "无":
                                    logger.info(f"跳过集成战略专属干员: {name} (获得方式: {obtain_method})")
                                    return None
                            
                            # 提取信息（基于arknights-toolkit的逻辑）
                            char_id = ID_PATTERN.search(content)
                            if not char_id:
                                return None
                            
                            # 星级在获取数据的基础上+1
                            base_rarity = int(RARITY_PATTERN.search(content).group(1)) if RARITY_PATTERN.search(content) else 3
                            rarity = base_rarity + 1
                            career = CHAR_PATTERN.search(content).group(1) if CHAR_PATTERN.search(content) else "未知"
                            subcareer = SUB_CHAR_PATTERN.search(content).group(1) if SUB_CHAR_PATTERN.search(content) else "未知"
                            race = RACE_PATTERN.search(content).group(1) if RACE_PATTERN.search(content) else "未知"
                            
                            # 提取组织信息（按优先级）
                            org = "未知"
                            for pattern in [ORG_PATTERN2, ORG_PATTERN1, ORG_PATTERN]:
                                match = pattern.search(content)
                                if match and match.group(1) != "无信息":
                                    org = match.group(1)
                                    break
                            
                            # 确定主组织和子组织
                            main_camp, sub_camp = determine_camp(org)
                            
                            artist = ART_PATTERN.search(content).group(1) if ART_PATTERN.search(content) else "未知"
                            
                            # 提取新增字段
                            operator_name = NAME_PATTERN.search(content).group(1) if NAME_PATTERN.search(content) else name
                            position = POSITION_PATTERN.search(content).group(1) if POSITION_PATTERN.search(content) else "未知"
                            
                            # 提取标签并分割
                            tags_raw = TAG_PATTERN.search(content).group(1) if TAG_PATTERN.search(content) else ""
                            tags = [tag.strip() for tag in tags_raw.split()] if tags_raw else []
                            tag1 = tags[0] if len(tags) > 0 else ""
                            tag2 = tags[1] if len(tags) > 1 else ""
                            tag3 = tags[2] if len(tags) > 2 else ""
                            tag4 = tags[3] if len(tags) > 3 else ""
                            
                            japanese_voice = JAPANESE_VOICE_PATTERN.search(content).group(1) if JAPANESE_VOICE_PATTERN.search(content) else "未知"
                            obtain_method = OBTAIN_METHOD_PATTERN.search(content).group(1) if OBTAIN_METHOD_PATTERN.search(content) else "未知"
                            online_time = ONLINE_TIME_PATTERN.search(content).group(1) if ONLINE_TIME_PATTERN.search(content) else "未知"
                            
                            # 提取属性信息
                            # 尝试多种部署费用字段
                            deploy_cost = "未知"
                            for pattern in [DEPLOY_COST_PATTERN, DEPLOY_COST_PATTERN2, DEPLOY_COST_PATTERN3, DEPLOY_COST_PATTERN4, DEPLOY_COST_PATTERN5]:
                                match = pattern.search(content)
                                if match:
                                    deploy_cost = match.group(1)
                                    break
                            
                            block_count = BLOCK_COUNT_PATTERN.search(content).group(1) if BLOCK_COUNT_PATTERN.search(content) else "未知"
                            attack_speed = ATTACK_SPEED_PATTERN.search(content).group(1) if ATTACK_SPEED_PATTERN.search(content) else "未知"
                            
                            # 智能提取最高阶段的满级属性
                            # 优先获取精英2属性，如果没有则获取精英1，最后获取精英0
                            elite2_hp = ELITE2_HP_PATTERN.search(content).group(1) if ELITE2_HP_PATTERN.search(content) else None
                            elite2_atk = ELITE2_ATK_PATTERN.search(content).group(1) if ELITE2_ATK_PATTERN.search(content) else None
                            elite2_def = ELITE2_DEF_PATTERN.search(content).group(1) if ELITE2_DEF_PATTERN.search(content) else None
                            elite2_magic_res = ELITE2_MAGIC_RES_PATTERN.search(content).group(1) if ELITE2_MAGIC_RES_PATTERN.search(content) else None
                            
                            elite1_hp = ELITE1_HP_PATTERN.search(content).group(1) if ELITE1_HP_PATTERN.search(content) else None
                            elite1_atk = ELITE1_ATK_PATTERN.search(content).group(1) if ELITE1_ATK_PATTERN.search(content) else None
                            elite1_def = ELITE1_DEF_PATTERN.search(content).group(1) if ELITE1_DEF_PATTERN.search(content) else None
                            elite1_magic_res = ELITE1_MAGIC_RES_PATTERN.search(content).group(1) if ELITE1_MAGIC_RES_PATTERN.search(content) else None
                            
                            elite0_hp = ELITE0_HP_PATTERN.search(content).group(1) if ELITE0_HP_PATTERN.search(content) else None
                            elite0_atk = ELITE0_ATK_PATTERN.search(content).group(1) if ELITE0_ATK_PATTERN.search(content) else None
                            elite0_def = ELITE0_DEF_PATTERN.search(content).group(1) if ELITE0_DEF_PATTERN.search(content) else None
                            elite0_magic_res = ELITE0_MAGIC_RES_PATTERN.search(content).group(1) if ELITE0_MAGIC_RES_PATTERN.search(content) else None
                            
                            # 确定最终使用的属性值（优先使用最高阶段）
                            final_hp = elite2_hp or elite1_hp or elite0_hp or "未知"
                            final_atk = elite2_atk or elite1_atk or elite0_atk or "未知"
                            final_def = elite2_def or elite1_def or elite0_def or "未知"
                            final_magic_res = elite2_magic_res or elite1_magic_res or elite0_magic_res or "未知"
                            
                            # 提取档案信息
                            gender = GENDER_PATTERN.search(content).group(1) if GENDER_PATTERN.search(content) else "未知"
                            birthplace = BIRTHPLACE_PATTERN.search(content).group(1) if BIRTHPLACE_PATTERN.search(content) else "未知"
                            birthday = BIRTHDAY_PATTERN.search(content).group(1) if BIRTHDAY_PATTERN.search(content) else "未知"
                            height = HEIGHT_PATTERN.search(content).group(1) if HEIGHT_PATTERN.search(content) else "未知"
                            # 技能1相关
                            skill1_name = SKILL1_NAME_PATTERN.search(content).group(1) if SKILL1_NAME_PATTERN.search(content) else (SKILL1_NAME_PATTERN2.search(content).group(1) if SKILL1_NAME_PATTERN2.search(content) else "未知")
                            skill1_type1 = SKILL1_TYPE1_PATTERN.search(content).group(1) if SKILL1_TYPE1_PATTERN.search(content) else (SKILL1_TYPE1_PATTERN2.search(content).group(1) if SKILL1_TYPE1_PATTERN2.search(content) else "未知")
                            skill1_type2 = SKILL1_TYPE2_PATTERN.search(content).group(1) if SKILL1_TYPE2_PATTERN.search(content) else (SKILL1_TYPE2_PATTERN2.search(content).group(1) if SKILL1_TYPE2_PATTERN2.search(content) else "未知")
                            # 使用智能提取函数获取技能专精3信息（除了描述）
                            skill1_spec3_initial = extract_skill_spec3_initial(content, 1)
                            skill1_spec3_cost = extract_skill_spec3_cost(content, 1)
                            skill1_spec3_duration = extract_skill_spec3_duration(content, 1)
                            
                            # 技能2相关
                            skill2_name = SKILL2_NAME_PATTERN.search(content).group(1) if SKILL2_NAME_PATTERN.search(content) else (SKILL2_NAME_PATTERN2.search(content).group(1) if SKILL2_NAME_PATTERN2.search(content) else "未知")
                            skill2_type1 = SKILL2_TYPE1_PATTERN.search(content).group(1) if SKILL2_TYPE1_PATTERN.search(content) else (SKILL2_TYPE1_PATTERN2.search(content).group(1) if SKILL2_TYPE1_PATTERN2.search(content) else "未知")
                            skill2_type2 = SKILL2_TYPE2_PATTERN.search(content).group(1) if SKILL2_TYPE2_PATTERN.search(content) else (SKILL2_TYPE2_PATTERN2.search(content).group(1) if SKILL2_TYPE2_PATTERN2.search(content) else "未知")
                            # 使用智能提取函数获取技能专精3信息（除了描述）
                            skill2_spec3_initial = extract_skill_spec3_initial(content, 2)
                            skill2_spec3_cost = extract_skill_spec3_cost(content, 2)
                            skill2_spec3_duration = extract_skill_spec3_duration(content, 2)
                            
                            # 技能3相关
                            skill3_name = SKILL3_NAME_PATTERN.search(content).group(1) if SKILL3_NAME_PATTERN.search(content) else (SKILL3_NAME_PATTERN2.search(content).group(1) if SKILL3_NAME_PATTERN2.search(content) else "未知")
                            skill3_type1 = SKILL3_TYPE1_PATTERN.search(content).group(1) if SKILL3_TYPE1_PATTERN.search(content) else (SKILL3_TYPE1_PATTERN2.search(content).group(1) if SKILL3_TYPE1_PATTERN2.search(content) else "未知")
                            skill3_type2 = SKILL3_TYPE2_PATTERN.search(content).group(1) if SKILL3_TYPE2_PATTERN.search(content) else (SKILL3_TYPE2_PATTERN2.search(content).group(1) if SKILL3_TYPE2_PATTERN2.search(content) else "未知")
                            # 使用智能提取函数获取技能专精3信息（除了描述）
                            skill3_spec3_initial = extract_skill_spec3_initial(content, 3)
                            skill3_spec3_cost = extract_skill_spec3_cost(content, 3)
                            skill3_spec3_duration = extract_skill_spec3_duration(content, 3)
                            
                            return {
                                "id": f"char_{char_id.group(1)}",
                                "name": operator_name,
                                "rarity": rarity,
                                "career": career,
                                "subcareer": subcareer,
                                "camp": main_camp,
                                "subcamp": sub_camp,
                                "race": race,
                                "artist": artist,
                                "position": position,
                                "tag1": tag1,
                                "tag2": tag2,
                                "tag3": tag3,
                                "tag4": tag4,
                                "japanese_voice": japanese_voice,
                                "obtain_method": obtain_method,
                                "online_time": online_time,
                                "deploy_cost": deploy_cost,
                                "block_count": block_count,
                                "attack_speed": attack_speed,
                                "max_hp": final_hp,
                                "max_atk": final_atk,
                                "max_def": final_def,
                                "max_magic_res": final_magic_res,
                                "gender": gender,
                                "birthplace": birthplace,
                                "birthday": birthday,
                                "height": height,
                                "skill1_name": skill1_name,
                                "skill1_type1": skill1_type1,
                                "skill1_type2": skill1_type2,
                                "skill1_spec3_initial": skill1_spec3_initial,
                                "skill1_spec3_cost": skill1_spec3_cost,
                                "skill1_spec3_duration": skill1_spec3_duration,
                                "skill2_name": skill2_name,
                                "skill2_type1": skill2_type1,
                                "skill2_type2": skill2_type2,
                                "skill2_spec3_initial": skill2_spec3_initial,
                                "skill2_spec3_cost": skill2_spec3_cost,
                                "skill2_spec3_duration": skill2_spec3_duration,
                                "skill3_name": skill3_name,
                                "skill3_type1": skill3_type1,
                                "skill3_type2": skill3_type2,
                                "skill3_spec3_initial": skill3_spec3_initial,
                                "skill3_spec3_cost": skill3_spec3_cost,
                                "skill3_spec3_duration": skill3_spec3_duration
                            }
        return None
        
    except Exception as e:
        logger.error(f"获取干员 {name} 信息失败: {e}")
        return None

def determine_camp(org: str) -> tuple[str, str]:
    """确定主组织和子组织（基于arknights-toolkit的逻辑）"""
    if not org or org == "未知":
        return "未知", "未知"
    
    # 特殊处理深池
    if org == "深池":
        return "维多利亚", "深池"
    
    # 根据组织关系确定主组织
    camp_mapping = {
        "罗德岛": "罗德岛",
        "行动组A4": "罗德岛",
        "行动预备组A1": "罗德岛",
        "行动预备组A4": "罗德岛",
        "行动预备组A6": "罗德岛",
        "S.W.E.E.P.": "罗德岛",
        "罗德岛-精英干员": "罗德岛",
        "龙门近卫局": "炎",
        "企鹅物流": "炎",
        "鲤氏侦探事务所": "炎",
        "黑钢国际": "哥伦比亚",
        "莱茵生命": "哥伦比亚",
        "汐斯塔": "哥伦比亚",
        "喀兰贸易": "谢拉格",
        "深海猎人": "阿戈尔",
        "红松骑士团": "卡西米尔",
        "格拉斯哥帮": "维多利亚",
        "塔拉": "维多利亚",
        "乌萨斯学生自治团": "乌萨斯",
        "贾维团伙": "叙拉古"
    }
    
    main_camp = camp_mapping.get(org, org)
    return main_camp, org

async def get_operator_list(client: httpx.AsyncClient) -> List[str]:
    """自动获取最新的干员列表"""
    logger.info("正在获取最新干员列表...")
    
    try:
        # 方法1：通过分类获取干员列表
        category_query = (
            f"{WIKI_API}?action=query&format=json&list=categorymembers"
            "&cmtitle=Category:干员&cmlimit=500&utf8=1"
        )
        
        response = await client.get(category_query)
        response.raise_for_status()
        data = response.json()
        
        if "query" in data and "categorymembers" in data["query"]:
            operators = []
            
            for index, item in enumerate(data["query"]["categorymembers"], 1):
                title = item["title"]
                # 移除"Category:"前缀
                if title.startswith("Category:"):
                    title = title[9:]
                
                # 过滤掉非干员页面
                if any(keyword in title for keyword in [
                    "密录", "剧情一览", "升级数值", "等级上限", 
                    "黑话", "梗", "成句", "资料相关", "特性一览",
                    "预备-", "spine", "编号相关", "首页", "亮点",
                    "卫星", "职业分支", "模组一览", "分支", "一览",
                    "预告", "公测前", "兑换券", "甄选", "邀请函", "装置",
                    "寻访模拟", "轮换卡池", "资深调用凭证", "凭证", "庆典",
                    "集成战略", "专属", "原型"  # 添加集成战略相关过滤
                ]):
                    continue
                
                # 如果页面名称包含"干员"，提取干员名称
                if "干员" in title:
                    operator_name = title.replace("干员", "").strip()
                    if operator_name and len(operator_name) <= 15:
                        operators.append(operator_name)
                # 如果页面名称不包含"干员"但看起来像干员名称
                elif (len(title) <= 15 and 
                      not any(char in title for char in ["/", "/", "\\"]) and
                      not any(keyword in title for keyword in ["真名", "海报", "凭证", "庆典"])):
                    operators.append(title)
                # 特殊处理：阿米娅的多形态（包含括号的职业标识）
                elif (len(title) <= 20 and 
                      "阿米娅(" in title and 
                      title.endswith(")") and
                      any(career in title for career in ["医疗", "近卫", "术师", "狙击", "重装", "先锋", "特种", "辅助"])):
                    operators.append(title)
                # 特殊处理：带中点的干员名称（如"维娜·维多利亚"）
                elif (len(title) <= 20 and 
                      "·" in title and
                      not any(char in title for char in ["/", "/", "\\"]) and
                      not any(keyword in title for keyword in ["真名", "海报", "凭证", "庆典", "spine", "语音记录"])):
                    operators.append(title)
            
            if operators:
                logger.success(f"通过分类API成功获取 {len(operators)} 个干员")
                return operators
        
        # 方法2：如果分类API失败，尝试搜索特定格式的干员页面
        search_query = (
            f"{WIKI_API}?action=query&format=json&list=search"
            "&srsearch=干员&srlimit=200&utf8=1"
        )
        
        response = await client.get(search_query)
        response.raise_for_status()
        data = response.json()
        
        if "query" in data and "search" in data["query"]:
            operators = []
            
            for index, item in enumerate(data["query"]["search"], 1):
                title = item["title"]
                
                # 过滤掉明显的非干员页面
                if any(keyword in title for keyword in [
                    "密录", "剧情一览", "升级数值", "等级上限", 
                    "黑话", "梗", "成句", "资料相关", "特性一览",
                    "预备-", "spine", "编号相关", "首页", "亮点",
                    "卫星", "职业分支", "模组一览", "分支", "一览",
                    "预告", "公测前", "兑换券", "甄选", "邀请函", "装置",
                    "寻访模拟", "轮换卡池", "资深调用凭证", "凭证", "庆典",
                    "集成战略", "专属", "原型"  # 添加集成战略相关过滤
                ]):
                    continue
                
                # 如果页面名称包含"干员"，提取干员名称
                if "干员" in title:
                    operator_name = title.replace("干员", "").strip()
                    if operator_name and len(operator_name) <= 15:
                        operators.append(operator_name)
                # 如果页面名称不包含"干员"但看起来像干员名称
                elif (len(title) <= 15 and 
                      not any(char in title for char in ["/", "/", "\\"]) and
                      not any(keyword in title for keyword in ["真名", "海报", "凭证", "庆典"])):
                    operators.append(title)
                # 特殊处理：阿米娅的多形态（包含括号的职业标识）
                elif (len(title) <= 20 and 
                      "阿米娅(" in title and 
                      title.endswith(")") and
                      any(career in title for career in ["医疗", "近卫", "术师", "狙击", "重装", "先锋", "特种", "辅助"])):
                    operators.append(title)
                # 特殊处理：带中点的干员名称（如"维娜·维多利亚"）
                elif (len(title) <= 20 and 
                      "·" in title and
                      not any(char in title for char in ["/", "/", "\\"]) and
                      not any(keyword in title for keyword in ["真名", "海报", "凭证", "庆典", "spine", "语音记录"])):
                    operators.append(title)
            
            if operators:
                logger.success(f"通过搜索API成功获取 {len(operators)} 个干员")
                return operators
        
        # 方法3：如果都失败了，尝试获取一些基础干员
        logger.warning("所有自动获取方法都失败了，使用基础干员列表")
        logger.info("尝试方法3：使用基础干员列表...")
        return []
        
    except Exception as e:
        logger.error(f"获取干员列表失败: {e}")
        return []

async def update_data():
    """更新数据"""
    global DATA_DIR, CAMP_FILE, CAREER_FILE, CHARACTERS_FILE
    logger.info("开始更新明日方舟猜谜游戏数据...")
    
    # 确保数据目录存在，若权限异常则回退到用户目录
    try:
        DATA_DIR.mkdir(parents=True, exist_ok=True)
    except PermissionError:
        fallback = Path.home() / ".arkguesser" / "data"
        logger.warning(f"数据目录无写权限，切换到 {fallback}")
        fallback.mkdir(parents=True, exist_ok=True)
        DATA_DIR = fallback
        CAMP_FILE = DATA_DIR / "camp.json"
        CAREER_FILE = DATA_DIR / "career.json"
        CHARACTERS_FILE = DATA_DIR / "characters.csv"
    
    async with httpx.AsyncClient(verify=False, timeout=30) as client:
        # 自动获取最新的干员列表
        operators = await get_operator_list(client)
        
        if not operators:
            logger.error("无法获取干员列表，更新终止")
            return False
        
        logger.info(f"开始获取 {len(operators)} 个干员的信息...")
        
        # 获取干员信息
        operator_data = []
        total_operators = len(operators)
        
        logger.info(f"开始获取 {total_operators} 个干员的信息...")
        
        for index, name in enumerate(operators, 1):
            info = await get_operator_info(name, client)
            if info:
                operator_data.append(info)
            # 只在每50个干员时显示进度，减少日志输出
            if index % 50 == 0:
                logger.info(f"已处理 {index}/{total_operators} 个干员...")
        
        logger.info(f"成功获取 {len(operator_data)} 个干员信息")
        
        if not operator_data:
            logger.error("没有获取到任何干员信息，更新终止")
            return False
        
        # 构建阵营数据
        camps = {}
        for op in operator_data:
            if op["camp"] not in camps:
                camps[op["camp"]] = []
            if op["subcamp"] not in camps[op["camp"]]:
                camps[op["camp"]].append(op["subcamp"])
        
        # 构建职业数据
        careers = {}
        for op in operator_data:
            if op["career"] not in careers:
                careers[op["career"]] = []
            if op["subcareer"] not in careers[op["career"]]:
                careers[op["career"]].append(op["subcareer"])
        
        # 保存文件
        logger.info("开始保存数据文件...")
        
        # 保存阵营数据
        with open(CAMP_FILE, 'w', encoding='utf-8-sig', newline='') as f:
            json.dump(camps, f, ensure_ascii=False, indent=2)
        logger.success(f"阵营数据已保存到 {CAMP_FILE}")
        
        # 保存职业数据
        with open(CAREER_FILE, 'w', encoding='utf-8-sig', newline='') as f:
            json.dump(careers, f, ensure_ascii=False, indent=2)
        logger.success(f"职业数据已保存到 {CAREER_FILE}")
        
        # 保存干员数据
        with open(CHARACTERS_FILE, 'w', newline='', encoding='utf-8-sig') as f:
            writer = csv.writer(f)
            
            # 写入表头
            headers = [
                'id', 'name', 'rarity', 'career', 'subcareer', 'camp', 'subcamp', 'race', 'artist',
                'position', 'tag1', 'tag2', 'tag3', 'tag4', 'japanese_voice', 'obtain_method', 'online_time',
                'deploy_cost', 'block_count', 'attack_speed', 'max_hp', 'max_atk', 'max_def', 'max_magic_res',
                'gender', 'birthplace', 'birthday', 'height',
                'skill1_name', 'skill1_type1', 'skill1_type2', 'skill1_spec3_initial', 'skill1_spec3_cost', 'skill1_spec3_duration',
                'skill2_name', 'skill1_type1', 'skill2_type2', 'skill2_spec3_initial', 'skill2_spec3_cost', 'skill2_spec3_duration',
                'skill3_name', 'skill3_type1', 'skill3_type2', 'skill3_spec3_initial', 'skill3_spec3_cost', 'skill3_spec3_duration'
            ]
            writer.writerow(headers)
            
            # 写入数据
            for index, op in enumerate(operator_data, 1):
                row = [
                    op["id"],           # id
                    op["name"],         # name
                    op["rarity"],       # rarity
                    op["career"],       # career
                    op["subcareer"],    # subcareer
                    op["camp"],         # camp
                    op["subcamp"],      # subcamp
                    op["race"],         # race
                    op["artist"],       # artist
                    op["position"],     # position
                    op["tag1"],         # tag1
                    op["tag2"],         # tag2
                    op["tag3"],         # tag3
                    op["tag4"],         # tag4
                    op["japanese_voice"], # japanese_voice
                    op["obtain_method"],  # obtain_method
                    op["online_time"],    # online_time
                    op["deploy_cost"],    # deploy_cost
                    op["block_count"],    # block_count
                    op["attack_speed"],   # attack_speed
                    op["max_hp"],
                    op["max_atk"],
                    op["max_def"],
                    op["max_magic_res"],
                    op["gender"],         # gender
                    op["birthplace"],     # birthplace
                    op["birthday"],       # birthday
                    op["height"],         # height
                    op["skill1_name"],    # skill1_name
                    op["skill1_type1"],   # skill1_type1
                    op["skill1_type2"],   # skill1_type2
                    op["skill1_spec3_initial"], # skill1_spec3_initial
                    op["skill1_spec3_cost"],   # skill1_spec3_cost
                    op["skill1_spec3_duration"], # skill1_spec3_duration
                    op["skill2_name"],    # skill2_name
                    op["skill2_type1"],   # skill2_type1
                    op["skill2_type2"],   # skill2_type2
                    op["skill2_spec3_initial"], # skill2_spec3_initial
                    op["skill2_spec3_cost"],   # skill2_spec3_cost
                    op["skill2_spec3_duration"], # skill2_spec3_duration
                    op["skill3_name"],    # skill3_name
                    op["skill3_type1"],   # skill3_type1
                    op["skill3_type2"],   # skill3_type2
                    op["skill3_spec3_initial"], # skill3_spec3_initial
                    op["skill3_spec3_cost"],   # skill3_spec3_cost
                    op["skill3_spec3_duration"] # skill3_spec3_duration
                ]
                writer.writerow(row)
        
        logger.success(f"干员数据已保存到 {CHARACTERS_FILE}")
        logger.info(f"总共保存了 {len(operator_data)} 个干员的数据")
        # 尝试刷新题库的星级统计缓存（pool_manager）
        try:
            # 优先使用包内相对导入（在包上下文中运行时）
            try:
                from ..game_tools.pool_manager import pool_manager  # type: ignore
            except Exception:
                # 退回到绝对导入（作为独立脚本运行时）
                from nonebot_plugin_arkguesser.game_tools.pool_manager import pool_manager  # type: ignore
            try:
                pool_manager.refresh_rarity_counts()
                logger.success("题库星级统计已刷新")
            except Exception as e:
                logger.warning(f"刷新题库星级统计失败: {e}")
        except Exception as e:
            logger.warning(f"无法导入题库管理器以刷新星级统计: {e}")

        logger.success("数据更新完成！")
        return True

async def main():
    """主函数"""
    try:
        success = await update_data()
        return 0 if success else 1
    except KeyboardInterrupt:
        logger.warning("用户中断更新过程")
        return 1
    except Exception as e:
        logger.error(f"更新过程中发生错误: {e}")
        return 1

if __name__ == "__main__":
    exit_code = asyncio.run(main())
    exit(exit_code)
